/*******************************************************************************
	Prepare coverage data used for reporting global poverty estimates

	- Part 1: Compute data coverage for PIP regions 
	- Part 2: Compute data coverage for country groups (FCV, income group, IDA) - using current classifications (fixed at the 2022 data year) 
	- Part 3: Compute data coverage for country groups (FCV, income group, IDA) - using evolving classifications  
	
	Written by: Samuel Kofi Tetteh Baah (stettehbaah@worldbank.org)
	Last date modified: 5 August 2024     
*******************************************************************************/

// Program set-up
clear all
set more off


///////////////////////////////////////////////////////////////////////////////////////
////////////////////// Compile country-year coverage data /////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////


// Load population data 
use "$ch1_data_processed/pop_reportinglevel", clear
gen j = 1 if reporting_level=="national"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="rural"
reshape wide pop, i(country_code year reporting_level) j(j)
collapse pop*,by(country_code year)
rename pop1 pop_nat 
rename pop2 pop_urb 
rename pop3 pop_rur
gen pop_urb_shr = 100*pop_urb/pop_nat
gen pop_rur_shr = 100*pop_rur/pop_nat

isid country_code year 

tempfile pop 
save `pop'

// Load the list of all distributions
use "$ch1_data_processed/poverty_country_survey.dta", clear
keep if poverty_line<3

merge m:1 country_code year using `pop'
sum _merge if _merge==1
sum year if _merge==1

keep if _merge==3
drop _merge
*br if country_code=="IDN" & poverty_line<3
drop if inlist(reporting_level,"urban","rural") & country_code=="IDN"
*br pop_urb_shr reporting_level country_code year if country_code=="BOL" & reporting_level=="urban"
*br pop_urb_shr reporting_level country_code year if reporting_level=="urban" & !inlist(country_code,"CHN","IND")
*sort pop_urb_shr
*br if country_code=="URY"
*br if country_code=="FSM"

// Remove surveys without national data coverage
*br if (reporting_level=="urban" & pop_urb_shr <50 & !inlist(country_code,"IND","CHN")) | (reporting_level=="rural" & pop_rur_shr <50 & !inlist(country_code,"IND","CHN")) | (inlist(reporting_level,"urban","rural") &!inlist(country_code,"IND","CHN","ARG","SUR"))


drop if (reporting_level=="urban" & pop_urb_shr <50 & !inlist(country_code,"IND","CHN")) | (reporting_level=="rural" & pop_rur_shr <50 & !inlist(country_code,"IND","CHN")) | (inlist(reporting_level,"urban","rural") &!inlist(country_code,"IND","CHN","ARG","SUR"))  // FSM-2000 and HND-1986, urban surveys from BOL, COL, ECU, rural Ethiopia

save "$ch1_data_processed/survey_distributions_national.dta", replace

// Calculate coverage -- precovid
drop _all
capture erase "$ch1_data_processed/coverage_precovid.dta"
save "$ch1_data_processed/coverage_precovid.dta", empty

use "$ch1_data_processed/survey_distributions_national.dta", clear
keep welfare_time country_code region_code
duplicates drop 
drop if welfare_time>=2019.5
gen j = _n
count 

forvalues i = 1/`r(N)'{
preserve 	
	keep if j==`i' 
	gen threshold_lo = welfare_time - 3
	gen threshold_hi = welfare_time + 3
	expand 56
	gen year = 1964 in 1
	replace year = year[_n-1] + 1 if year==.
	drop if year>threshold_hi | year<threshold_lo
	keep country_code year

	append using "$ch1_data_processed/coverage_precovid.dta"
	duplicates drop 
	sort country_code year 
	
	save "$ch1_data_processed/coverage_precovid.dta", replace
restore 
} 

// Calculate coverage -- covid years
drop _all
capture erase "$ch1_data_processed/coverage_covid.dta"
save "$ch1_data_processed/coverage_covid.dta", empty

use "$ch1_data_processed/survey_distributions_national", clear
keep welfare_time country_code region_code 
duplicates drop 
drop if welfare_time<2019.5
gen j = _n
count 

forvalues i = 1/`r(N)'{
preserve 	
	keep if j==`i' 
	gen threshold_lo = welfare_time - 3
	gen threshold_hi = welfare_time + 3
	expand 3
	gen year = 2020 in 1
	replace year = year[_n-1] + 1 if year==.
	drop if year>threshold_hi | year<threshold_lo
	keep country_code year

	append using "$ch1_data_processed/coverage_covid.dta"
	duplicates drop 
	sort country_code year 
	
	save "$ch1_data_processed/coverage_covid.dta", replace
restore 
} 


///////////////////////////////////////////////////////////////////////////////////////
////////////////////// Coverage results for PIP regions ///////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

// Prepare data on income classification of countries
use "$ch1_data_raw/classifications/CLASS.dta", clear 
keep code year_data incgroup_historical
rename code country_code 
rename year_data year 

tempfile class
save `class'

// Load country list
use "$ch1_data_processed/countrylist.dta", clear
keep country_code region_code country_name africa_split_code
rename region_code wbregioncode

tempfile countrylist 
save `countrylist'

*Load population data
use "$ch1_data_processed/pop_reportinglevel", clear
gen j = 1 if reporting_level=="national"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="rural"
keep country_code year pop reporting_level

merge m:1 country_code using `countrylist', keep(match) nogen 
keep if year<=2023

gen j = 1 if reporting_level=="rural"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="national" 

keep if reporting_level=="national" | inlist(country_code,"ARG") & reporting_level=="urban"
keep if year<=2023
egen year_max = max(year),by(country_code)
sum year_max
sum  
drop year_max 

egen double pop_nat_ = mean(pop) if reporting_level=="national",by(country_code year)
egen double pop_nat = mean(pop_nat_),by(country_code year)

sort country_code year 
drop if reporting_level=="national" & inlist(country_code,"ARG") 

egen id = group(country_code)
sum id
drop pop_nat_ j id 
sum country_code year

tempfile pop 
save `pop'

// Combine coverage data 
use "$ch1_data_processed/coverage_precovid.dta", clear 
append using "$ch1_data_processed/coverage_covid.dta"

gen coverage = 1
gen reporting_level = "national"
replace reporting_level = "urban" if inlist(country_code,"ARG")

merge 1:1 country_code year reporting_level using `pop', nogen 
merge 1:1 country_code year using `class', nogen


preserve 
	keep if inlist(africa_split_code,"AFE","AFW")
	replace wbregioncode = africa_split_code
	
	gen subregion = 1

	tempfile afr 
	save `afr'
restore 

append using `afr'

keep if year<=2022
egen year_max = max(year),by(country_code)
*tab year_max
sum 

sort country_code year 
*br 
duplicates drop 
*sum 
*br if year!=. & pop==.
**tab year if year!=. & pop==.

replace subregion = 0 if subregion==.
isid country_code year subregion

// Calculate regional population by year 
egen double reg_pop = sum(pop_nat),by(wbregioncode year)
replace coverage = 0 if coverage==.
*tab coverage
count 
gen coverage_pop = coverage*pop 

// Calculate regional coverage
preserve 
	egen double reg_pop_coverage = sum(coverage_pop),by(wbregioncode year)
	gen double  COV = 100*reg_pop_coverage/reg_pop
	collapse (mean) COV,by(wbregioncode year) 

	tempfile reg 
	save `reg'
restore 

// Calculate global coverage
drop if subregion == 1
egen double glo_pop = sum(pop_nat),by(year)
egen double glo_pop_coverage = sum(coverage_pop),by(year)
gen double lmic = (inlist(incgroup_historical,"Low income","Lower middle income"))
*tab lmic 
gen double lmic_coverage = lmic*coverage_pop
egen double glo_lmic_coverage = sum(lmic_coverage),by(year)
egen double glo_pop_lmic = sum(pop_nat) if lmic==1,by(year)
gen double  COV = 100*glo_pop_coverage/glo_pop
gen double COV_LMIC =  100*glo_lmic_coverage/glo_pop_lmic 
collapse (mean) COV (mean) COV_LMIC,by(year)
gen wbregioncode = "WLD"
append using `reg'
*br if wbregioncode == "MNA"
*br if wbregioncode == "AFE" | wbregioncode=="AFW"
rename wbregioncode region_code
drop if COV==. & COV_LMIC==.
keep if inrange(year,1981,2022)
order region_code year COV*
rename COV cov 
rename COV_LMIC cov_LMIC
lab var cov "Share of population with data coverage (%)"
lab var cov_LMIC "Share of population in LIC/LMIC with data coverage (%)"
save "$ch1_data_processed/data_coverage_pip_regions.dta", replace



///////////////////////////////////////////////////////////////////////////////////////
////// Coverage results for country groups using fixed, current classification ////////
///////////////////////////////////////////////////////////////////////////////////////

// Prepare data on classification of countries

* Income group
use "$ch1_data_raw/classifications/CLASS.dta", clear
keep code incgroup_current
rename incgroup_current group 
gen type = "inc_group"
duplicates drop 
rename code country_code 
count 

tempfile inc_group
save `inc_group'

* FCV
use "$ch1_data_raw/classifications/CLASS.dta", clear
keep code fcv_current
gen group = "FCV" if fcv_current=="Yes"
replace group = "Non-FCV" if fcv_current=="No"
drop fcv_current
duplicates drop 
drop if group==""
gen type = "fcv_group"
rename code country_code 
count 

tempfile fcv_group
save `fcv_group'


* IDA 
use "$ch1_data_raw/classifications/CLASS.dta", clear
*br if code=="USA" | code=="GHA"
keep code ida_current
gen group = "IDA" if inlist(ida_current,"IDA","Blend")
replace group = "IBRD" if inlist(ida_current,"IBRD")
replace group = "Rest of the World" if inlist(ida_current,"Rest of the world")
drop ida_current
duplicates drop
*br if group==""
drop if group==""
gen type = "ida_group"
rename code country_code 
count 

tempfile ida_group
save `ida_group'

* Small states
import excel "$ch1_data_raw/classifications/FY25 small states.xlsx", sheet("composition") firstrow clear
*br if WB_Group_Code=="SST"
keep if WB_Group_Code=="SST"
keep WB_Group_Code WB_Country_Code
rename WB_Country_Code country_code 
rename WB_Group_Code group 
gen type = "sst_group"


tempfile sst_group
save `sst_group'

// Combine groups 
use `ida_group', clear 
append using `fcv_group'
append using `inc_group'
append using `sst_group'

tempfile country_groups 
save `country_groups'

use "$ch1_data_processed/countrylist.dta", clear
keep country_code region_code country_name 
rename region_code wbregioncode

tempfile countrylist 
save `countrylist'

// Load population data
use "$ch1_data_processed/pop_reportinglevel", clear
gen j = 1 if reporting_level=="national"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="rural"
keep country_code year pop reporting_level
merge m:1 country_code using `countrylist', keep(match) nogen 
keep if year<=2022

gen j = 1 if reporting_level=="rural"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="national" 

keep if reporting_level=="national" | inlist(country_code,"ARG") & reporting_level=="urban"
keep if year<=2023
egen year_max = max(year),by(country_code)
sum year_max
sum  
drop year_max 

egen double pop_nat_ = mean(pop) if reporting_level=="national",by(country_code year)
egen double pop_nat = mean(pop_nat_),by(country_code year)

sort country_code year 
drop if reporting_level=="national" & inlist(country_code,"ARG") 

egen id = group(country_code)
sum id
drop pop_nat_ j id 
sum country_code year

tempfile pop 
save `pop'

// Combine coverage data 
use "$ch1_data_processed/coverage_precovid.dta", clear 
append using "$ch1_data_processed/coverage_covid.dta"
gen coverage = 1
gen reporting_level = "national"
replace reporting_level = "urban" if inlist(country_code,"ARG")
merge 1:1 country_code year reporting_level using `pop', nogen 

expand 3
bysort country_code year reporting_level: gen j = _n
gen type = "ida_group" if j==1
replace type = "inc_group" if j==2
replace type = "fcv_group" if j==3
replace type = "sst_group" if j==4
drop j
merge m:1 country_code type using `country_groups', nogen
drop if group==""
sum year 

keep if year<=2022
egen year_max = max(year),by(country_code)
*tab year_max
sum 

sort country_code year 
*br 
duplicates drop 
sum 
*br if year!=. & pop==.
*tab year if year!=. & pop==.
isid country_code year type

// Calculate regional population by year 
egen double reg_pop = sum(pop_nat),by(group type year)
replace coverage = 0 if coverage==.
*tab coverage
count 
gen coverage_pop = coverage*pop 

// Calculate regional coverage
egen double reg_pop_coverage = sum(coverage_pop),by(group type year)
gen double  COV = 100*reg_pop_coverage/reg_pop
collapse (mean) COV,by(group type year)
rename COV cov_fixed
keep year group cov_fixed
*line cov_fixed year, sort by(group) yline(50)

tempfile 1
save `1'



///////////////////////////////////////////////////////////////////////////////////////
/////////// Coverage results for country groups using evolving classification /////////
///////////////////////////////////////////////////////////////////////////////////////

// Prepare data on classification of countries

* Income group
use "$ch1_data_raw/classifications/CLASS.dta", clear
keep code year_data incgroup_historical
rename year_data year
rename incgroup_historical group 
gen type = "inc_group"
duplicates drop 
rename code country_code 
count 

tempfile inc_group
save `inc_group'

* FCV
use "$ch1_data_raw/classifications/CLASS.dta", clear
keep code year_data fcv_historical
rename year_data year
gen group = "FCV" if fcv_historical=="Yes"
replace group = "Non-FCV" if fcv_historical=="No"
drop fcv_historical
drop if group==""
gen type = "fcv_group"
rename code country_code 
count 

tempfile fcv_group
save `fcv_group'


* IDA 
use "$ch1_data_raw/classifications/CLASS.dta", clear
keep code year_data ida_historical
rename year_data year
gen group = "IDA" if inlist(ida_historical,"IDA","Blend")
replace group = "IBRD" if inlist(ida_historical,"IBRD")
replace group = "Rest of the World" if inlist(ida_historical,"Rest of the world")
drop ida_historical
drop if group==""
gen type = "ida_group"
rename code country_code 
count 

tempfile ida_group
save `ida_group'

// Combine groups 
use `ida_group', clear 
append using `fcv_group'
append using `inc_group'

tempfile country_groups 
save `country_groups'

// Load country list
use "$ch1_data_processed/countrylist.dta", clear
keep country_code region_code country_name africa_split_code
rename region_code wbregioncode

tempfile countrylist 
save `countrylist'

// Load population data
use "$ch1_data_processed/pop_reportinglevel", clear
gen j = 1 if reporting_level=="national"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="rural"
keep country_code year pop reporting_level

merge m:1 country_code using `countrylist', keep(match) nogen 
keep if year<=2022

gen j = 1 if reporting_level=="rural"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="national" 
drop africa_split_code

keep if reporting_level=="national" | inlist(country_code,"ARG") & reporting_level=="urban"
keep if year<=2022
egen year_max = max(year),by(country_code)
sum year_max
sum  
drop year_max 

egen double pop_nat_ = mean(pop) if reporting_level=="national",by(country_code year)
egen double pop_nat = mean(pop_nat_),by(country_code year)

sort country_code year 
drop if reporting_level=="national" & inlist(country_code,"ARG") 

egen id = group(country_code)
sum id
drop pop_nat_ j id 
sum country_code year

tempfile pop 
save `pop'

// Combine coverage data 
use "$ch1_data_processed/coverage_precovid.dta", clear 
append using "$ch1_data_processed/coverage_covid.dta"
gen coverage = 1
gen reporting_level = "national"
replace reporting_level = "urban" if inlist(country_code,"ARG")
merge 1:1 country_code year reporting_level using `pop', nogen 

expand 3
bysort country_code year reporting_level: gen j = _n
gen type = "ida_group" if j==1
replace type = "inc_group" if j==2
replace type = "fcv_group" if j==3
drop j
merge 1:1 country_code year type using `country_groups', nogen
drop if group==""
sum year 

keep if year<=2022
egen year_max = max(year),by(country_code)
*tab year_max
sum 

sort country_code year 
*br 
duplicates drop 
sum 
*br if year!=. & pop==.
*tab year if year!=. & pop==.
isid country_code year type

// Calculate regional population by year 
egen double reg_pop = sum(pop_nat),by(group type year)
replace coverage = 0 if coverage==.
**tab coverage
count 
gen coverage_pop = coverage*pop 

// Calculate regional coverage
egen double reg_pop_coverage = sum(coverage_pop),by(group type year)
gen double  COV = 100*reg_pop_coverage/reg_pop
collapse (mean) COV,by(group type year)

keep if inrange(year,1987,2022)
*line COV year, sort by(group) yline(50)
rename COV cov_evolving 
keep year group cov_evolving

// Combine results for both fixed and evolving classifications 
merge 1:1 group year using `1'
sort group year
drop if cov_evolving==. & cov_fixed==.
keep if inrange(year,1981,2022)
drop _merge  

// Visualize results
*preserve 
*drop if inlist(group,"Non-FCV","Rest of the World","IBRD")
*line cov_fixed cov_evolving year, sort by(group, row(2) legend(position(6))) yline(50) legend(position(6)) ///
 legend(order(1 "Fixed 2022 classification" 2 "Evolving classification")) ytitle("Coverage (%)")
*restore 
// Label variabes 
lab var cov_fixed "Population share with data (%), fixed 2022 classification"
lab var cov_evolving "Population share with data (%), evolving classification"
order group year cov*

// Store results
save "$ch1_data_processed/data_coverage_country_groups.dta", replace

// Clean up folder
erase "$ch1_data_processed/coverage_precovid.dta"
erase "$ch1_data_processed/coverage_covid.dta"
erase "$ch1_data_processed/survey_distributions_national.dta"