
/*******************************************************************************
	Compute share of population with survey data in 2020 or later, comparing PSPR 2022 and PSPR 2024

	- Part 1: Organize country groups
	- Part 2: Compute data coverage for country groups (FCV, income group, IDA) - using current classifications (fixed at the 2022 data year) 
	- Part 3: Compute data coverage for country groups (FCV, income group, IDA) - using evolving classifications  
	
	Written by: Samuel Kofi Tetteh Baah (stettehbaah@worldbank.org)
	Last date modified: 6 August 2024     
*******************************************************************************/


///////////////////////////// Country groups //////////////////////////////////
use "$ch1_data_raw/classifications/CLASS.dta", clear
keep code year_data region_pip 
rename year_data year
rename region_pip group
gen type = "PIP"
tempfile 1 
save `1'

use "$ch1_data_raw/classifications/CLASS.dta", clear
keep code year_data incgroup_historical 
rename year_data year
rename incgroup_historical group
gen type = "Income"
tempfile 2 
save `2'


use "$ch1_data_raw/classifications/CLASS.dta", clear
keep code year_data fcv_historical 
rename year_data year
drop if fcv_historical==""
replace fcv_historical = "FCV" if fcv_historical=="Yes"
replace fcv_historical = "Non-FCV" if fcv_historical=="No"
rename fcv_historical group
gen type = "FCV"
tempfile 3 
save `3'

use "$ch1_data_raw/classifications/CLASS.dta", clear
keep code year_data ida_historical 
rename year_data year
drop if ida_historical==""
replace ida_historical = "IDA" if inlist(ida_historical,"IDA","Blend") 
replace ida_historical = "Non-IDA" if !inlist(ida_historical,"IDA","Blend")
rename ida_historical group
gen type = "IDA"
tempfile 4 
save `4'

// Combine data
use `1', clear 
append using `2'
append using `3'
append using `4'

tab type if year==2020
rename code country_code
compress
save "$ch1_data_processed/country_groups.dta", replace


/////////////////////////////// Country-year coverage in 2020 or later - PSPR 2022 ////////////////////////////////////

drop _all
capture erase "$ch1_data_processed/coverage_pspr_2022_covid.dta"
save "$ch1_data_processed/coverage_pspr_2022_covid.dta", empty

// Load PIP data
use "$ch1_data_processed/pip_data_pspr22.dta", clear 
drop if welfare_time<2019.5
duplicates drop 
gen j = _n
count 
forvalues i = 1/`r(N)'{
preserve 	
	keep if j==`i' 
	gen threshold_lo = welfare_time - 3
	gen threshold_hi = welfare_time + 3
	expand 4
	gen year = 2020 in 1
	replace year = year[_n-1] + 1 if year==.
	drop if year>threshold_hi | year<threshold_lo
	keep country_code year

	append using "$ch1_data_processed/coverage_pspr_2022_covid.dta"
	duplicates drop 
	sort country_code year 
	
	save "$ch1_data_processed/coverage_pspr_2022_covid.dta", replace
restore 
} 


// Prepare coverage data 
use "$ch1_data_processed/coverage_pspr_2022_covid.dta", clear 
gen reporting_level = "national"
replace reporting_level = "urban" if country_code=="ARG" 
gen coverage = 1
tempfile df 
save `df'


// Combine coverage data with population numbers
use "$ch1_data_processed/pop_reportinglevel", clear
gen j = 1 if reporting_level=="national"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="rural"
drop j

merge 1:1 country_code year reporting_level using `df'
keep if _merge==3 | _merge==1
drop _merge 
keep if year>=2020
sort country_code year reporting_level
keep if reporting_level=="national" | inlist(country_code,"ARG")
drop if reporting_level=="national" & inlist(country_code,"ARG")
sort country_code year reporting_level

// Check that Argentina and Suriname have only rural and urban sectors for all years since 1990
bysort country_code year (reporting_level): gen j = _n 
egen count_year = group(year)
sum count_year  // the total count of years
tab j if inlist(country_code,"ARG")
tab j if !inlist(country_code,"ARG")  // all other countries should have only 1 observation per year 

isid country_code year reporting_level 

egen count_country = group(country_code)
sum count_country // there should be 218 countries 
drop j count_*
replace coverage = 0 if coverage==.
save "$ch1_data_processed/coverage_country_level_pspr_2022_clean.dta", replace


/////////////////////////////// Country-year coverage in 2020 or later - PPPR 2024 ////////////////////////////////////


/// First, prepare country-level coverage data  /// 
drop _all
capture erase "$ch1_data_processed/coverage_pppr_2024_covid.dta"
save "$ch1_data_processed/coverage_pppr_2024_covid.dta", empty

// Load PIP data
*pip, server(dev) version(20240627_2017_01_02_PROD) clear 
use "$ch1_data_raw/pip/pip_data_survey_202408.dta", clear 
keep welfare_time country_code 
drop if welfare_time<2019.5
duplicates drop 
gen j = _n
count 
forvalues i = 1/`r(N)'{
preserve 	
	keep if j==`i' 
	gen threshold_lo = welfare_time - 3
	gen threshold_hi = welfare_time + 3
	expand 4
	gen year = 2020 in 1
	replace year = year[_n-1] + 1 if year==.
	drop if year>threshold_hi | year<threshold_lo
	keep country_code year

	append using "$ch1_data_processed/coverage_pppr_2024_covid.dta"
	duplicates drop 
	sort country_code year 
	
	save "$ch1_data_processed/coverage_pppr_2024_covid.dta", replace
restore 
} 


// Prepare coverage data  
use "$ch1_data_processed/coverage_pppr_2024_covid.dta", clear 
gen reporting_level = "national"
replace reporting_level = "urban" if country_code=="ARG" 
gen coverage = 1
isid country_code year reporting_level 
tempfile df 
save `df'


// Combine coverage data with population numbers
use "$ch1_data_processed/pop_reportinglevel", clear
gen j = 1 if reporting_level=="national"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="rural"
drop j

merge 1:1 country_code year reporting_level using `df'
keep if _merge==3 | _merge==1
drop _merge 
keep if year>=2020
sort country_code year reporting_level
keep if reporting_level=="national" | inlist(country_code,"ARG")
drop if reporting_level=="national" & inlist(country_code,"ARG")
*br if inlist(country_code,"ARG","SUR") & year==2020
sort country_code year reporting_level

// Check that Argentina has only rural and urban sectors for all years 
bysort country_code year (reporting_level): gen j = _n 
egen count_year = group(year)
sum count_year  // the total count of years
*tab j if inlist(country_code,"ARG","SUR")
*tab j if !inlist(country_code,"ARG","SUR")  // all other countries should have only 1 observation per year 

isid country_code year reporting_level 

egen count_country = group(country_code)
sum count_country // there should be 218 countries 
drop j count_*
replace coverage = 0 if coverage==.
save "$ch1_data_processed/coverage_country_level_pppr_2024_clean.dta", replace


/////////////////////////////// Summarize coverage by country groups in 2020 or later  ////////////////////////////////////


// Compute coverage data 
foreach var in pspr_2022 pppr_2024{
use "$ch1_data_processed/coverage_country_level_`var'_clean.dta", clear

drop if year>2022
drop if year<2020
expand 5
bysort country_code year reporting_level: gen j = _n 
gen type = "Income" if j==1 
replace type = "FCV" if j==2
replace type = "IDA" if j==3
replace type = "PIP" if j==4
replace type = "WLD" if j==5

merge m:1 country_code year type using "$ch1_data_processed/country_groups.dta"
drop if pop==.
replace group = "WLD" if type=="WLD"
sum year if _merge==1
drop if group == ""

// Simply select the latest year 
keep if year==2022
egen pop_grp = sum(pop),by(group year type)

gen cov_pop_grp = coverage*pop
egen cov_pop_grp_all = sum(cov_pop_grp),by(group year type)

gen coverage_result = 100*cov_pop_grp_all/ pop_grp
keep year coverage_result group type
duplicates drop 
*br if year==2020

sort coverage_result
drop if inlist(group,"Rest of the world","IBRD")
sort group 

preserve 
keep if type=="Income"
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 hic 
rename cov2 lic 
rename cov3 lmic
rename cov4 umic 

tempfile df1 
save `df1'
restore

preserve 
keep if type=="PIP"
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 eap 
rename cov2 eca 
rename cov3 lac
rename cov4 mna
rename cov5 ohi 
rename cov6 sas 
rename cov7 ssa

tempfile df2 
save `df2'
restore


preserve 
drop if year<1998
drop if inlist(type,"PIP","Income","WLD")
drop if group=="Non-IDA"
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 fcv 
rename cov2 ida  
rename cov3 nfcv 

tempfile df3 
save `df3'
restore


preserve 
keep if inlist(type,"WLD")
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 wld  

tempfile df4 
save `df4'
restore

// Combine data sets
use `df1', clear 
append using `df2'
append using `df3'
append using `df4'

collapse lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv wld,by(year)
sort year 

save "$ch1_data_processed/coverage_`var'_results_groups.dta", replace
}


/////////////////////////////// Organize results data  ////////////////////////////////////


// PSPR 2022
use "$ch1_data_processed/coverage_pspr_2022_results_groups.dta", clear 
local j = 1
foreach var in lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv wld{
	rename `var' y`j'
	local j = `j'+1
}

reshape long y, i(year) j(j)

gen type = ""
local j = 1
foreach var in lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv wld{
replace type = "`var'" if j==`j'
	local j = `j'+1
}

replace type = upper(type)
drop if type == "NFCV"
drop j
rename y coverage 
rename type group 
gen type = "pspr_2022" 

tempfile pspr_2022
save `pspr_2022'


* PPPR 2024
use "$ch1_data_processed/coverage_pppr_2024_results_groups.dta", clear 
local j = 1
foreach var in lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv wld{
	rename `var' y`j'
	local j = `j'+1
}

reshape long y, i(year) j(j)

gen type = ""
local j = 1
foreach var in lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv wld{
replace type = "`var'" if j==`j'
	local j = `j'+1
}

replace type = upper(type)
drop if type == "NFCV"
drop j
rename y coverage 
rename type group 
gen type = "pppr_2024" 

tempfile pppr_2024 
save `pppr_2024'

// Combine coverage data from PSPR 2022 and PPPR 2024
use `pspr_2022', clear 
append using `pppr_2024'

save "$ch1_data_processed/data_coverage_pspr_pppr.dta", replace 


// Clean up folder 
erase "$ch1_data_processed/country_groups.dta" 
erase "$ch1_data_processed/coverage_pspr_2022_covid.dta" 
erase "$ch1_data_processed/coverage_country_level_pspr_2022_clean.dta" 
erase "$ch1_data_processed/coverage_country_level_pppr_2024_clean.dta"
erase "$ch1_data_processed/coverage_pppr_2024_covid.dta"
erase "$ch1_data_processed/coverage_pppr_2024_results_groups.dta"
erase "$ch1_data_processed/coverage_pspr_2022_results_groups.dta"