
/*******************************************************************************
	Estimate rural, urban, and total poverty rates by region

	/* Part 1: Aggregate total poverty rates across regions */
	/* Part 2: Aggregate rural / urban poverty rates across regions */
	/* Part 3: Combine data */ 
	
	Written by: Samuel Kofi Tetteh Baah (stettehbaah@worldbank.org)
	Last date modified: 10 August 2024     
*******************************************************************************/

/*  Aggregate total poverty rates across regions */
use "$ch1_data_processed/countrylist", clear
keep pcn_region_code country_code
tempfile ctylist 
save `ctylist'

// Population data
use "$ch1_data_processed/pop_reportinglevel", clear
keep if year==2022
keep if reporting_level=="national" | country_code=="ARG"
drop if country_code=="ARG" & inlist(reporting_level,"rural","national")
isid country_code
count 

tempfile pop 
save `pop' 

// Load global distribution
use "$ch1_data_raw/other/global_profile_lineup_2022.dta", clear
keep country_code gallT_ppp _newweight urban
rename gallT_ppp welf_ppp 
rename _newweight weight

gen double poor215 = (welf_ppp<=2.15)
gen double poor685 = (welf_ppp<=6.85)

// Combine data sets
merge m:1 country_code using `pop', nogen keep(match)
merge m:1 country_code using `ctylist', nogen

// Drop missing observations in the urban variable
drop if !inlist(urban,0,1)
rename pcn_region region
egen double wgt_tot = sum(weight),by(country_code)
gen double weight_pop = pop*weight/wgt_tot
egen pop_check = sum(weight_pop),by(country_code)
*br country_code pop_check pop 
sum pop_check pop 
egen pop_wld = sum(weight_pop)
sum 
drop pop 

preserve 
	egen hc215 = wtmean(poor215), weight(weight_pop)
	egen hc685 = wtmean(poor685), weight(weight_pop)
	egen pop = sum(weight_pop)
	keep hc215 hc685  pop
	duplicates drop 
	gen region = "WLD"

	tempfile wld 
	save `wld'
restore 

bysort region: egen hc215 = wtmean(poor215), weight(weight_pop)
bysort region: egen hc685 = wtmean(poor685), weight(weight_pop)
egen pop = sum(weight_pop),by(region)
keep hc215 hc685 pop region

duplicates drop 
append using `wld'
keep hc* region pop
duplicates drop 

save "$ch1_data_raw/other/temp_global_poor_total.dta", replace 


/*  Aggregate rural / urban poverty rates across regions */
use "$ch1_data_processed/countrylist", clear
keep pcn_region_code country_code
tempfile ctylist 
save `ctylist'

// Population data
use "$ch1_data_processed/pop_reportinglevel", clear
keep if year==2022
keep if reporting_level=="national" | country_code=="ARG"
drop if country_code=="ARG" & inlist(reporting_level,"rural","national")
isid country_code
count 

tempfile pop 
save `pop'

// Load global distribution
use "$ch1_data_raw/other/global_profile_lineup_2022.dta", clear
count 
keep country_code gallT_ppp _newweight urban
rename gallT_ppp welf_ppp 
rename _newweight weight
gen double poor215 = (welf_ppp<=2.15)
gen double poor685 = (welf_ppp<=6.85)

// Combine data sets
merge m:1 country_code using `pop', nogen keep(match)
merge m:1 country_code using `ctylist', nogen

// Drop missing observations in the urban variable
drop if !inlist(urban,0,1) 
rename pcn_region region

egen double wgt_tot = sum(weight),by(country_code)
gen double weight_pop = pop*weight/wgt_tot
egen pop_check = sum(weight_pop),by(country_code)
*br country_code pop_check pop 
sum pop_check pop 
egen pop_wld = sum(weight_pop)
sum 
drop pop 

preserve 
	bysort urban: egen hc215 = wtmean(poor215), weight(weight_pop)
	bysort urban: egen hc685 = wtmean(poor685), weight(weight_pop)
	egen pop = sum(weight_pop),by(urban)
	keep urban hc215 hc685  pop
	duplicates drop 
	gen region = "WLD"

	tempfile wld 
	save `wld'
restore 

bysort region urban: egen hc215 = wtmean(poor215), weight(weight_pop)
bysort region urban: egen hc685 = wtmean(poor685), weight(weight_pop)
egen pop = sum(weight_pop),by(region urban)
keep urban hc215 hc685  pop  region

duplicates drop 
append using `wld'

save "$ch1_data_raw/other/temp_global_poor_urbrur.dta", replace 

/* Combine data */
use "$ch1_data_raw/other/temp_global_poor_urbrur.dta", clear 
append using "$ch1_data_raw/other/temp_global_poor_total.dta"
replace urban = 2 if urban==.
label define urban_lbl 1 "Urban" 0 "Rural" 2 "Total",  replace 
label values urban urban_lbl
rename region region_code 
rename hc215 hc_a 
rename hc685 hc_b 
lab var hc_a "Percent of population living below $2.15 a day"
lab var hc_b "Percent of population living below $6.85 a day"
compress
save "$ch1_data_processed/global_poor_location.dta", replace 

// Clean up folder 
erase "$ch1_data_raw/other/temp_global_poor_total.dta" 
erase "$ch1_data_raw/other/temp_global_poor_urbrur.dta"
