
/*******************************************************************************
	Annex 1D. Regional and global poverty estimates, 1990 – 2024

	Written by: Samuel Kofi Tetteh Baah (stettehbaah@worldbank.org)
	Last date modified: 8 August 2024     
*******************************************************************************/

// Program set-up
clear all
set more off

// Prepare data on population coverage 
use "$ch1_data_processed/data_coverage_country_groups.dta", clear
rename group region_code 
keep region_code year cov_fixed 
rename cov_fixed cov 

tempfile 1
save `1'

use "$ch1_data_processed/data_coverage_pip_regions.dta", clear
append using `1'

tempfile coverage 
save `coverage'


// Make country groups 

// IDA
use "$ch1_data_processed/countrycodes_yearly.dta", clear
keep ida_current code 
duplicates drop 
rename ida_current ida 
replace ida = "IDA" if ida=="Blend"
keep if ida=="IDA" 
rename code country_code 
count
tempfile ida 
save `ida' 

// FCV
use "$ch1_data_processed/countrycodes_yearly.dta", clear
keep fcv_current code 
duplicates drop  
rename fcv_current fcv 
replace fcv = "FCV" if fcv=="Yes"
keep if fcv=="FCV" 
rename code country_code 
count
tempfile fcv 
save `fcv' 

// Load lined-up data 
use "$ch1_data_processed/poverty_country_lineup.dta", clear 
drop if poverty_line>24
keep country_code year poverty_line headcount reporting_level
rename headcount hc  

merge m:1 country_code year reporting_level using "$ch1_data_processed/pop_reportinglevel.dta", nogen 
keep if reporting_level=="national" | country_code=="ARG"
drop if reporting_level=="national" & country_code=="ARG"

*Create observations for countries with missing data for all three poverty lines
expand 3 if hc==.
bysort country_code year reporting_level: gen poverty_line_id = _n
replace poverty_line = 2.15 if poverty_line_id==1 & hc==.
replace poverty_line = 3.65 if poverty_line_id==2 & hc==.
replace poverty_line = 6.85 if poverty_line_id==3 & hc==.

merge m:1 country_code  using "$ch1_data_processed/countrylist.dta", nogen  
merge m:1 country_code  using `fcv', nogen 
merge m:1 country_code  using `ida', nogen 
drop world_code world region_code region poverty_line_id
rename pcn_region region_name 
rename pcn_region_code region_code 
 
// Get Africa split
preserve 
	keep if inlist(africa_split_code,"AFW","AFE")
	replace region_code = africa_split_code
	gen subregion = 1
	replace region_name = africa_split

	tempfile subreg 
	save `subreg'
restore 

append using `subreg'

// Estimate regional poverty estimates
bysort poverty_line region_code year: egen hc_reg_avg = wtmean(hc),weight(pop)

egen hc_reg_avg_ssa_ = mean(hc_reg_avg) if region_code=="SSA",by(poverty_line region_code year)
egen hc_reg_avg_ssa = mean(hc_reg_avg_ssa_),by(poverty_line year)
replace hc_reg_avg = hc_reg_avg_ssa if subregion==1

// Assign countries with missing data regional poverty rate
replace hc = hc_reg_avg if missing(hc)&!missing(hc_reg_avg) 

// Estimate global poverty
preserve 
drop if subregion==1    // Remove duplicates of African countries
	collapse (mean) hc (rawsum) pop [aw=pop],by(poverty_line year)
	gen region_code = "WLD"
	gen region_name = "World"

	tempfile wld 
	save `wld'
restore 

// Estimate poverty for FCV countries
preserve 
drop if subregion==1    // Remove duplicates of African countries
keep if fcv=="FCV"
		collapse (mean) hc (rawsum) pop [aw=pop],by(poverty_line year)
	gen region_code = "FCV"
	gen region_name = "FCV"

	tempfile fcv 
	save `fcv'
restore 

// Estimate poverty for IDA countries
preserve 
drop if subregion==1    // Remove duplicates of African countries
keep if ida=="IDA"
		collapse (mean) hc (rawsum) pop [aw=pop],by(poverty_line year)
	gen region_code = "IDA"
	gen region_name = "IDA"

	tempfile ida 
	save `ida'
restore 

// Estimate regional poverty rates
collapse (mean) hc (rawsum) pop [aw=pop],by(poverty_line region_code region_name year)

// Pool together regional and global poverty estimates
append using `wld'
append using `fcv'
append using `ida'

// Compare with numbers in PIP 
merge 1:1 region_code year poverty_line using "$ch1_data_processed/poverty_aggregates.dta", keep(1 3)
gen double d_hc = round(abs(hc - headcount),0.001)
gsort -d_hc 
*br if d_hc>0 & d_hc!=.
*sum d_hc
*assert `r(mean)'==0

*br if inlist(region_code,"WLD","SSA") & year ==2022

keep year poverty_line region_name region_code hc pop estimate_type 
gen double poor = hc*pop/1000000 
replace hc = 100*hc 
egen poor_total_ = mean(poor) if region_code=="WLD",by(poverty_line year) 
egen poor_total = mean(poor_total_),by(poverty_line year) 
drop poor_total_ pop
gen double share_poor = 100*poor/poor_total
 
// Merge with coverage data 
merge m:1 region_code year using `coverage', keep(1 3)
keep if inlist(year,1990,1995,2000,2005,2010,2015) | inlist(year,2019,2020,2021,2022,2023,2024)

// For the world, 50% of the world should have data coverage AND 50% percent of low- and lower-middle-income countries should have coverage
egen cov_min = rowmin(cov cov_LMIC)
replace cov = cov_min if region_code=="WLD"

/* See section 2 of the following document:
https://documents1.worldbank.org/curated/en/099839303252425642/pdf/IDU1d671646616eef14bb31a2ba103042c40ae3c.pdf?_gl=1*1x5mn5e*_gcl_au*MjY1MTgwMDkxLjE3MjI0NTA2MzU.

(NB: last full stop is part of the link)
*/

*br region_code cov hc year if (inlist(region_code,"MNA","SSA","AFE","AFW","IDA","FCV") & inrange(year,2018,2022)) & poverty_line<3
  
// Censor observation with insufficient coverage 
replace hc = . if cov <50 & !inlist(year,2023,2024) & !(inlist(region_code,"MNA","SSA","AFE","AFW","IDA","FCV") & inrange(year,2019,2022)) 
replace poor = . if cov <50 & !inlist(year,2023,2024) & !(inlist(region_code,"MNA","SSA","AFE","AFW","IDA","FCV") & inrange(year,2019,2022)) 
replace share_poor = . if cov <50 & !inlist(year,2023,2024) & !(inlist(region_code,"MNA","SSA","AFE","AFW","IDA","FCV") & inrange(year,2019,2022))

// Organize data 
drop cov_LMIC cov_min _merge cov estimate_type poor_total
replace region_name="Eastern and Southern Africa" if region_code=="AFE"
replace region_name="Western and Central Africa" if region_code=="AFW"
replace region_name="Middle East and North Africa" if region_code=="MNA"
replace region_name = "Rest of the World" if region_code=="OHI"
replace region_name = "East Asia and Pacific" if region_code=="EAP"
replace region_name = "Latin America and the Caribbean" if region_code=="LAC"
replace region_name = "Europe and Central Asia" if region_code=="ECA"

rename hc poverty_rate
rename poor millions_poor 
reshape wide poverty_rate millions_poor share_poor, i(region_code region_name poverty_line) j(year)
replace region_code = "SSA_AFW" if region_code=="AFW"
replace region_code = "SSA_AFE" if region_code=="AFE"
replace region_code = "TOHI" if region_code=="OHI"
replace region_code = "XFCV" if region_code=="FCV"
replace region_code = "YIDA" if region_code=="IDA"
replace region_name = "FCS" if region_code=="XFCV"
sort poverty_line region_code 
drop region_code

// Save results 
/* Poverty rate */
preserve 
keep poverty_line region_name poverty_rate*
export excel using "$ch1_output/chapter1.xlsx", sheet("tab 1d1", modify) firstrow(variables) 
restore 

/* Millions of poor */
preserve 
keep poverty_line region_name millions_poor*
export excel using "$ch1_output/chapter1.xlsx", sheet("tab 1d2", modify) firstrow(variables) sheetreplace
restore 

/* Share of global poor */
preserve 
keep poverty_line region_name share_poor*
export excel using "$ch1_output/chapter1.xlsx", sheet("tab 1d3", modify) firstrow(variables) sheetreplace
restore 
