
/*******************************************************************************
	Figure 1C.1: Concentration of extreme poverty in SSA and FCS under different poverty rates for India in 2022

	Written by: Nishant Yonzan 
	Last date modified: 12 August 2024     
*******************************************************************************/


// Load data
use "$ch1_data_processed/projections_country_2030.dta", clear

// Keep relevant data
keep if year==2022 & povertyline==2.15
gen 	group = 1 if fcvFY24=="Yes" & region_pip=="SSA"
replace group = 2 if fcvFY24=="No" & region_pip=="SSA"
replace group = 3 if fcvFY24=="Yes" & region_pip!="SSA"
replace group = 4 if fcvFY24=="No" & region_pip!="SSA"

// Generate scenario 
forval i = 0(1)12 {
	gen 	poverty_rate`i' = poverty_rate
	replace poverty_rate`i' = `i' if code=="IND"
	
	gen pop_in_poverty`i' = poverty_rate`i'/100 * population
}

collapse (rawsum) pop_in_poverty*, by(group)
drop pop_in_poverty

// Generage the share of global poor for each group
forval i = 0(1)12 {
	egen globalpoor`i' = total(pop_in_poverty`i')
	gen  poorshare`i' = pop_in_poverty`i'/globalpoor`i' * 100
}

keep group poorshare*
reshape long poorshare, i(group) j(scenario)
reshape wide poorshare, i(scenario) j(group)

gen  fcs_ssa = poorshare1 
egen nfc_ssa = rowtotal(poorshare1 poorshare2)	
egen fcs_oth = rowtotal(poorshare1 poorshare2 poorshare3)
egen rest 	 = rowtotal(poorshare1 poorshare2 poorshare3 poorshare4)

gsort -scenario 
gen sortvar = _n

// Make chart
tw 	(area rest fcs_oth nfc_ssa fcs_ssa sortvar, ///
	color("$color4" "$color3" "$color2" "$color1")) ///
	(scatteri 0 1.4 100 1.4, recast(line) color(black) lw(thick)) ///
	(scatter rest sortvar, ms(none) yaxis(2)), ///
	text(50 1.6 "India poverty rate in 2022", orient(vertical)) ///
	xtitle("Various poverty rates for India") ///
	ytitle("Share of global poor in 2022, %", axis(1)) ///
	ytitle("Share of global poor in 2022, %", axis(2)) ///
	ylabel(0(20)100, axis(1)) ylabel(0(20)100, axis(2)) ///
	xlab(1 "12%" 3 "10%" 5 "8%" 7 "6%" 9 "4%" 11 "2%" 13 "0%") ///
	legend(pos(6) row(1) order(4 "FCS in SSA" 3 "Non-FCS in SSA" 2 "FCS outside SSA" 1 "Rest"))

// Save chart
graph export "$ch1_output/ch1_fig1c1_GlobalPoverty_India_Scenario.eps", as(eps) replace
graph export "$ch1_output/ch1_fig1c1_GlobalPoverty_India_Scenario.png", as(png) replace 

export excel using "$ch1_output/chapter1.xlsx", sheet("fig1c1", modify) firstrow(variables) sheetreplace
