
/*******************************************************************************
	Figure 8: Economic growth has been an important driver of extreme poverty reduction

	Written by: Samuel Kofi Tetteh Baah (stettehbaah@worldbank.org)
	Last date modified: 6 August 2024     
*******************************************************************************/

// Program set-up
clear all
set more off


// Load data on income classification 
use "$ch1_data_processed/countrycodes_yearly.dta", clear
keep economy code year_data incgroup_historical
rename year_data year 
rename incgroup_historical incgroup 
keep if year==1990
keep if incgroup=="Low-income"
rename code country_code
count 
keep country_code 
duplicates drop 

tempfile df 
save `df'

// Load survey data
*pip, clear server(dev) version(20240627_2017_01_02_PROD)
use "$ch1_data_processed/poverty_country_survey.dta", clear
keep if poverty_line<3
*br if country_code=="COL" & (year==1988 | year==2000)  // not comparable because of different reporting level
keep country_code year comparable_spell welfare_type welfare_time headcount reporting_level population region_name country_name mean
rename population pop
drop if inlist(reporting_level,"rural","urban") & inlist(country_code,"CHN","IND","IDN")
gen double year1 = real(substr(comparable_spell,1,4))
gen double year2 = real(substr(comparable_spell,8,4))
*br if country_code=="IDN"
*br if year2==.
drop if year2==.  // these surveys are not comparable with other surveys
drop if year1<1980

// Select longest comparable spells
keep if year==year1 | year==year2
gen d_year = year2 - year1
drop if year2==2023
egen d_year_max = max(d_year),by(country_code reporting_level)
keep if d_year == d_year_max
bysort country_code welfare_type reporting: gen j = _N
*tab j
drop if j==1
*br if j!=2
sort country_code year
drop if country_code=="COL" & year==2000  
drop year1 year2 d_year j d_year_max 
isid country_code welfare_type reporting_level year
bysort country_code year: gen dup = _N 
*tab dup 
*br if dup==2
drop dup
drop if country_code=="PHL" & welfare_type==2
isid country_code year

// Combine data sets
merge m:1 country_code using `df', keep(match) nogen
bysort country_code comparable_spell: gen dup = _N
*tab dup // All should be 2
*br if country_code=="GHA"
drop welfare_type 
rename headcount hc 
rename welfare_time wt 
bysort country_code comparable_spell (year): gen j = _n
reshape wide hc wt year pop mean, i(country_code comparable_spell) j(j)

// Compute annualized growth rate
gen d_mean = mean2/mean1
gen year_diff = wt2-wt1
gen gr = (d_mean^(1/year_diff))
replace gr = (gr - 1)*100

// Compute annualized change in poverty 
gen hc_chg = (hc2/hc1)^(1/year_diff)
replace hc_chg = (hc_chg - 1)*100

// Decide which spell to drop when two spells are of equal length
*br if country_code=="IDN"
drop if year1==1993 & country_code=="IDN" // keep older survey
drop if year2==2021 & inlist(country_code,"NER","MLI")  // keep older survey
isid country_code

// Clean up and organize data  
keep gr hc_chg pop2 country_code country_name region_name 
drop if country_code=="EGY"  // drop Egypt which is an outlier

egen j = group(region_name)
tab region_name
rename pop2 pop
reshape wide gr hc_chg, i(country_name region_name country_code) j(j) 
egen hc_chg = rowmean(hc_chg*)
egen gr = rowmean(gr*)

// Make chart
drop if country_code=="MMR"  // drop Myanmar which is an outlier
twoway scatter gr1 hc_chg1 [aw=pop], msymbol(circle_hollow) mcolor("237 52 46") msize(medium) ///
	|| scatter gr2 hc_chg2 [aw=pop], msymbol(circle_hollow) mcolor("58 190 203") msize(medium) ///
	|| scatter gr3 hc_chg3 [aw=pop], msymbol(circle_hollow) mcolor("255 129 51") msize(medium) ///
	|| scatter gr4 hc_chg4 [aw=pop], msymbol(circle_hollow) mcolor("255 213 137") msize(medium) ///
	ytitle("Growth in per capita consumption (%)", size(small)) xtitle("Change in extreme poverty (%)", size(small))   ///
 legend(order(1 "East Asia and Pacific" 2 "Latin America and the Caribbean"   ///
	3 "South Asia" 4 "Sub-Saharan Africa") row(2) symxsize(*.5) symysize(*.8) size(small))  ///
	legend(position(6)) xsize(8) ysize(6) ylab(,labsize(small)) xlab(,labsize(small))

	
// Export graph in .eps format
graph export "$ch1_output/ch1_fig8_economic_growth.eps", as(eps) replace
graph export "$ch1_output/ch1_fig8_economic_growth.png", as(png) replace 

keep country_code region_name country_name pop hc_chg gr 
lab var pop "Population"
lab var hc_chg "Annualized change in poverty rate, %"
lab var gr "Annualized change in per capita consumption, %"
lab var country_code "Country code"
lab var country_name "Country name"
lab var region_name "Region name"
export excel "$ch1_output/chapter1.xlsx", sheet("fig8") firstrow(varlabels) sheetreplace

