
********************************************************************************
*	Figure 9. Incomes have grown over time but a small negative income or consumption shock can push a large share of the population in low-income countries back into poverty 
********************************************************************************
use "$ch1_data_processed/country_income_distribution.dta", clear
* Merge in missing countries and population
merge m:1 country_code reporting_level year using "$ch1_data_processed/pop_reportinglevel"
drop 	_merge
rename 	country_code code
* Merge country classifications
merge 	m:1 code using "$ch1_data_processed/countrycodes_current.dta",		///
				 keepusing(region_pip incgroup_current) nogen
keep	if year == 1990 | year == 2024
* Create one observation in missing data for each poverty line
fillin code year poverty_line reporting_level
* Clean up 
keep if reporting_level=="national" | inlist(code,"ARG") 												// Drop urban and rural for all but Argentina
drop if inlist(code,"ARG") & reporting_level=="national" 												// Drop nationall for Argentina	* Calculate regional poverty to fill in missing numbers
bys    code year reporting_level (_fillin): carryforward pop region_pip incgroup_current, replace		// Copy region and population numbers for all country-year obs
drop 	_fillin
drop if poverty_line == .
* Calculate regional poverty to fill in missing numbers
bysort region_pip poverty_line year: egen avg_pov = wtmean(headcount),weight(population)
* Fill in missing values
replace population = pop if population == . 															// Get population
replace headcount  = avg_pov if headcount == . 	
gen 	pop_in_poverty = headcount * population
* Gen distribution of income (non-cumulative)
bys code year reporting_level (poverty_line) : gen pop_in_poverty2 = pop_in_poverty - pop_in_poverty[_n-1]	// Reporting level because of Argentina
replace pop_in_poverty = pop_in_poverty2 if pop_in_poverty2 !=.
* Aggregate to income group level
collapse (sum) pop_in_poverty population, by(incgroup_current poverty_line year)
bys poverty_line year: egen global_pop = sum(population)
gen pop_share_welf = pop_in_poverty / global_pop
* Excel Format
gen 	incg = 1 if incgroup_current == "Low-income" 													// Numeric for order
replace incg = 2 if incgroup_current == "Lower-middle-income" 
replace incg = 3 if incgroup_current == "Upper-middle-income" 
replace incg = 4 if incgroup_current == "High-income"
drop incgroup_current population
drop pop_in_poverty global_pop
reshape wide pop_share_welf, i(year poverty_line) j(incg)

label var pop_share_welf1 "Low-income" 
label var pop_share_welf2 "Lower-middle-income"
label var pop_share_welf3 "Upper-middle-income"
label var pop_share_welf4 "High-income"
order poverty_line pop_share_welf* year

********** Create graphs in Stata 
format %3.0f poverty_line
gen stack_pop_share_welf1 = pop_share_welf1
gen stack_pop_share_welf2 = stack_pop_share_welf1 + pop_share_welf2
gen stack_pop_share_welf3 = stack_pop_share_welf2 + pop_share_welf3
gen stack_pop_share_welf4 = stack_pop_share_welf3 + pop_share_welf4
label var stack_pop_share_welf1 "Low-income" 
label var stack_pop_share_welf2 "Lower-middle-income"
label var stack_pop_share_welf3 "Upper-middle-income"
label var stack_pop_share_welf4 "High-income"
* 1990 
twoway area stack_pop_share_welf1 poverty_line 									///
			if year == 1990, fcolor("$color3") lcolor("$color3") ||				///
	   rarea stack_pop_share_welf1 stack_pop_share_welf2 poverty_line 			///
			if year == 1990, fcolor("$color2") lcolor("$color2") ||				///
	   rarea stack_pop_share_welf2 stack_pop_share_welf3 poverty_line 			///
			if year == 1990, fcolor("$color4") lcolor("$color4") ||				///
	   rarea stack_pop_share_welf3 stack_pop_share_welf4 poverty_line 			///
			if year == 1990, fcolor("$color7") lcolor("$color7") ||				///
	   (scatteri 0 2.15 0.14 2.15, recast(line) lcolor(black*0.4) lpattern(-))	///
	   (scatteri 0 3.65 0.14 3.65, recast(line) lcolor(black*0.4) lpattern(-))	///
	   (scatteri 0 6.85 0.14 6.85, recast(line) lcolor(black*0.4) lpattern(-))	///
	   (scatteri 0 2.94 0.133 2.94, recast(line) lcolor("black") lpattern(l))	///		// 2.94 is the median per capita daily income in 1990
	   (pcarrowi  0.08 8 0.08 3),												///
	   legend(order(1 "Low-income" 2 "Lower-middle-income" 						///
					3 "Upper-middle-income" 4 "High-income")					///
			  row(2) pos(6) size(12pt)) xsc(titlegap(1))		 				///
			  xtitle("Income or consumption per person ($ per day)", size(12pt))						///
			  ylabel(0 "0%" 0.02 "2%" 0.04 "4%" 0.06 "6%" 0.08 "8%" 0.10 "10%"  ///
			  0.12 "12%" 0.14 "14%", labsize(12pt))								///
			  xlabel(0(2)25, labsize(12pt)) 									///
			  ytitle("Share of global population")								///
			  text(0.14 1.2 "2.15" 0.14 4.6 "3.65" 0.14 7.8 "6.85" 				///
			  0.08 13 "Half of the world lived on" "less than $2.94 per day", 	///
			  size(12pt))
graph export "$ch1_output/ch1_fig9a.png", replace
graph export "$ch1_output/ch1_fig9a.eps", replace	
preserve 
	keep if year == 1990
	keep stack* year
 	export excel "$ch1_output/chapter1.xlsx", sheet("fig9a", modify) firstrow(varlabels) cell(A1)	
restore

* 2024			  
twoway area stack_pop_share_welf1 poverty_line 									///
			if year == 2024, fcolor("$color3") lcolor("$color3") ||				///
	   rarea stack_pop_share_welf1 stack_pop_share_welf2 poverty_line 			///
			if year == 2024, fcolor("$color2") lcolor("$color2") ||				///
	   rarea stack_pop_share_welf2 stack_pop_share_welf3 poverty_line 			///
			if year == 2024, fcolor("$color4") lcolor("$color4") ||				///
	   rarea stack_pop_share_welf3 stack_pop_share_welf4 poverty_line 			///
			if year == 2024, fcolor("$color7") lcolor("$color7") ||				///
	   (scatteri 0 2.15 0.14 2.15, recast(line) lcolor(black*0.4) lpattern(-))	///
	   (scatteri 0 3.65 0.14 3.65, recast(line) lcolor(black*0.4) lpattern(-))	///
	   (scatteri 0 6.85 0.14 6.85, recast(line) lcolor(black*0.4) lpattern(-))	///
	   (scatteri 0 8.24 0.133 8.24, recast(line) lcolor("black") lpattern(l))	///		// 8.24 is the median per capita daily income in 1990
	   (pcarrowi  0.08 10 0.08 8.3),											///	   
	   legend(order(1 "Low-income" 2 "Lower-middle-income" 						///
					3 "Upper-middle-income" 4 "High-income") 					///
			  row(2) pos(6) size(12pt)) xsc(titlegap(1)) 						///
			  xtitle("Income or consumption per person (US$ per day)", size(12pt))						///
			  ylabel(0 "0%" 0.02 "2%" 0.04 "4%" 0.06 "6%" 0.08 "8%" 0.10 "10%" 	///
			  0.12 "12%" 0.14 "14%", labsize(12pt))								///
			  xlabel(0(2)25, labsize(12pt)) 									///
			  ytitle("Share of global population")								///
			  text(0.14 1.2 "2.15" 0.14 4.6 "3.65" 0.14 7.8 "6.85" 				///
			  0.08 15 "Half of the world lives on" "less than $8.24 per day", ///
			  size(12pt))
graph export "$ch1_output/ch1_fig9b.png", replace
graph export "$ch1_output/ch1_fig9b.eps", replace	

preserve 
	keep if year == 2024
	keep stack* year
	export excel "$ch1_output/chapter1.xlsx", sheet("fig9b", modify) firstrow(varlabels) cell(A1)	
restore
