
/*******************************************************************************
	Figure B1.4.2: Share of global poor in 2022 by region

	Written by: Samuel Kofi Tetteh Baah (stettehbaah@worldbank.org)
	Last date modified: 6 August 2024     
*******************************************************************************/

// Obtain population data 
use "$ch1_data_processed/pop_reportinglevel", clear
gen j = 1 if reporting_level=="national"
replace j = 2 if reporting_level=="urban"
replace j = 3 if reporting_level=="rural"

tempfile pop
save `pop'

*Obtain country and regional list 
use "$ch1_data_processed/countrycodes_yearly.dta", clear 
keep region_pip region_pip_name code 
rename code country_code
rename region_pip_name region_name  
rename region_pip region_code
duplicates drop
isid country_code

tempfile countrylist 
save `countrylist'

// 2.15
use "$ch1_data_processed/poverty_country_lineup.dta", clear
keep if poverty_line<3
keep if year==2022
sort region_code year
keep country_code reporting_level welfare_type year headcount 
isid country_code year reporting_level
rename headcount hc1  

tempfile 1
save `1'

// 3.65
use "$ch1_data_processed/poverty_country_lineup.dta", clear
drop if poverty_line<3
keep if poverty_line<4
tab poverty_line
keep if year==2022
sort region_code year
keep country_code reporting_level welfare_type year headcount 
isid country_code year reporting_level
rename headcount hc2  

tempfile 2
save `2'

// 6.85
use "$ch1_data_processed/poverty_country_lineup.dta", clear
drop if poverty_line<5
keep if poverty_line<7
tab poverty_line
keep if year==2022
sort region_code year
keep country_code reporting_level welfare_type year headcount 
isid country_code year reporting_level
rename headcount hc3

tempfile 3
save `3'

*Obtain reference-year poverty estimates
use "$ch1_data_processed/poverty_country_lineup.dta", clear
keep if year==2022
sort region_code year
keep country_code reporting_level welfare_type year  spr
duplicates drop
isid country_code year reporting_level
rename spr hc4  


*Combine data sets
merge 1:1 country_code year reporting_level using `1', nogen
merge 1:1 country_code year reporting_level using `2', nogen
merge 1:1 country_code year reporting_level using `3', nogen
merge 1:1 country_code year reporting_level using `pop', nogen 
merge m:1 country_code using `countrylist', nogen  // Merge with country and regional list
keep if reporting_level=="national" | inlist(country_code,"ARG") 
keep if year==2022

*Estimate regional poverty estimates
forvalues i=1/4{
	bysort region_code year: egen hc_reg_hc`i' = wtmean(hc`i'),weight(pop)
}


*Assign countries with missing data regional poverty rate
forvalues i=1/4{
	replace hc`i' = hc_reg_hc`i' if missing(hc`i')&!missing(hc_reg_hc`i') 
}


drop if inlist(country_code,"ARG") & reporting_level=="national"

*Estimate global poverty
preserve 
	collapse (mean) hc1 (mean) hc2  (mean) hc3 (mean) hc4  (rawsum) pop [aw=pop],by(year)
	gen region_code = "WLD"
	gen region_name = "World"

	tempfile wld 
	save `wld'
restore 

*Estimate regional poverty rates
collapse (mean) hc1 (mean) hc2  (mean) hc3  (mean) hc4 (rawsum) pop [aw=pop],by(region_code region_name year)

*Pool together regional and global poverty estimates
append using `wld'
forvalues i = 1/4{
	gen poor`i' = hc`i' * pop/1000000
}

keep poor* region_code region_name year

// Regional sum of poor should match global poor
forvalues i = 1/4{
egen poor`i'_sum = sum(poor`i') if region_code!="WLD"
}
 
drop poor*_sum

// Reshape data
reshape long poor, i(region_code region_name year) j(j)
gen type = "$2.15" if j==1
replace type = "$3.65" if j==2 
replace type = "$6.85" if j==3 
replace type = "SPL" if j==4
drop j
bysort year type (region_code): gen j = _n
reshape wide poor, i(region_code type year region_name) j(j)
collapse poor*,by( type )

forvalues i = 1/7{
	replace poor`i' = 100*poor`i'/poor8
}
drop poor8

graph set window fontface "Open Sans"

// Make chart
graph hbar poor1 poor2 poor3 poor4 poor6 poor7 poor5, over(type) stack ///
bar(1, lpattern(solid) lcolor(237 52 46)  fcolor(237 52 46) fintensity(inten100))  ///
	bar(2, lpattern(solid) lcolor(163 163 163)  fcolor(163 163 163) fintensity(inten100))  ///
	bar(3, lpattern(solid) lcolor(58 190 203)  fcolor(58 190 203) fintensity(inten100))  ///
	bar(4, lpattern(solid) lcolor(14 100 169)  fcolor(14 100 169) fintensity(inten100))  ///
	bar(5, lpattern(solid) lcolor(255 129 51)  fcolor(255 129 51) fintensity(inten100))  ///
	bar(6, lpattern(solid) lcolor(255 213 137)  fcolor(255 213 137) fintensity(inten100))  ///
	bar(7, lpattern(solid) lcolor(153 245 255)  fcolor(153 245 255) fintensity(inten100))  ///
	legend(order(1 "East Asia and Pacific" ///  
2 "Europe and Central Asia" ///
3 "Latin America and the Caribbean" /// 
4 "Middle East and North Africa" /// 
5 "South Asia" /// 
6 "Sub-Saharan Africa"   /// 
7 "Rest of the World") /// 
position(6) row(4) symxsize(*.5) symysize(*.8) size(small)) ///
ytitle("Share of global poor (%)") xsize(9) ysize(8)


// Export graph in .eps format
graph export "$ch1_output/ch1_figB1.4.2_share_poor.eps", as(eps) replace
graph export "$ch1_output/ch1_figB1.4.2_share_poor.png", as(png) replace 

// Save results in Excel
lab var poor1 "East Asia and Pacific"
lab var poor2 "Europe and Central Asia"
lab var poor3 "Latin America and the Caribbean"
lab var poor4 "Middle East and North Africa"
lab var poor5 "Rest of the World"
lab var poor6 "South Asia"
lab var poor7 "Sub-Saharan Africa"

export excel "$ch1_output/chapter1.xlsx", sheet("figB1.4.2") firstrow(varlabels) sheetreplace

