
/*******************************************************************************
	Table 1: Create MPM table for regions, world, FCV and IDA countries

	Written by: Samuel Kofi Tetteh Baah (stettehbaah@worldbank.org)
	Last date modified: 7 August 2024     
*******************************************************************************/



// FCV
use "$ch1_data_processed/countrycodes_yearly.dta", clear
keep code fcv_current 
duplicates drop 
rename fcv_current fcv 
replace fcv = "FCV" if fcv=="Yes"
replace fcv = "Non-FCV" if fcv!="FCV"
rename code country_code
count 

tempfile fcv 
save `fcv'

// IDA
use "$ch1_data_processed/countrycodes_yearly.dta", clear
keep code ida_current 
duplicates drop 
rename ida_current ida 
replace ida = "IDA" if ida=="Blend"
replace ida = "Non-IDA" if ida!="IDA"
rename code country_code
count 

tempfile ida 
save `ida'



// Country list
use "$ch1_data_processed/countrycodes_yearly.dta", clear
keep region_pip region_pip_name code 
rename region_pip_name 	region_name 
rename region_pip 		region_code 
rename code 			country_code
duplicates drop
isid country_code
tempfile countrylist 
save `countrylist'


// Population
use "$ch1_data_processed/pop_reportinglevel", clear 
keep if year==2021
drop year

tempfile pop 
save `pop' 

// Load MPM data 
use "$ch1_data_processed/mpm_3pr24.dta", clear 
keep code dep_* mpm datayear
rename datayear year
rename code country_code 
gen reporting_level = "national"
*replace reporting_level = "urban" if country_code=="ARG"
isid country_code
merge 1:m country_code reporting_level using `pop', gen(merge)
merge m:1 country_code using `countrylist', nogen 
merge m:1 country_code using `ida', nogen
merge m:1 country_code using `fcv', nogen

egen double pop_fcv = sum(pop) if reporting_level=="national",by(fcv)
egen double pop_ida = sum(pop) if reporting_level=="national",by(ida)
egen double pop_reg = sum(pop) if reporting_level=="national",by(region_code)
egen double pop_wld = sum(pop) if reporting_level=="national"

// Now keep only countries with surveys
keep if merge==3  
count
br if region_code=="SAS"
egen double pop_fcv_ = sum(pop) if year>2019,by(fcv)
egen double pop_ida_ = sum(pop) if year>2019,by(ida)
egen double pop_reg_ = sum(pop) if year>2019,by(region_code)
egen double pop_wld_ = sum(pop) if year>2019


// Count economies
tab region_code 
gen count = 1


// FCV
preserve 
collapse dep* mpm pop_fcv pop_fcv_ (rawsum) count  [aw=pop],by(fcv)
rename fcv region_name
rename pop_fcv pop_reg
rename pop_fcv_ pop_reg_

tempfile fcv 
save `fcv'
restore


// IDA
preserve 
collapse dep* mpm pop_ida pop_ida_ (rawsum) count  [aw=pop],by(ida)
rename ida region_name
rename pop_ida pop_reg
rename pop_ida_ pop_reg_

tempfile ida 
save `ida'
restore

// World
preserve 
collapse dep* mpm pop_wld pop_wld_ (rawsum) count  [aw=pop]
rename pop_wld pop_reg
rename pop_wld_ pop_reg_

gen region_code = "WLD"
gen region_name = "World"
tempfile wld 
save `wld'
restore 

// Combine results
collapse dep* mpm pop_reg pop_reg_ (rawsum) count  [aw=pop],by(region_code region_name)
append using `wld'
append using `ida'
append using `fcv'

gen double cov = 100*pop_reg_/pop_reg
drop region_code 
drop pop*
drop dep_all_inds dep_1_dims dep_2_dims dep_3_dims

foreach var in dep_monetarypoor dep_educ_com dep_educ_enr dep_infra_elec dep_infra_imps dep_infra_impw2  mpm {
	replace `var' = 100*`var'
}
drop if region_name=="Non-IDA" | region_name=="Non-FCV"
br region_name cov

// Save in results in output folder
export excel using "$ch1_output/chapter1.xlsx", sheet("tab1") firstrow(variables) sheetreplace
