********************************************************************************
/*
Gini dataset for 3PR
*/
********************************************************************************

********************************************************************************
* Step 1: Data Prep: Gini Coefficients from PIP
********************************************************************************
use "${ch2_input_processed}pip_3pr2024_v20240802.dta", clear 											
keep if poverty_line == 2.15													// keep one observations per survey
keep if reporting_level=="national" | inlist(country_code,"ARG")          		// ARG has urban poverty rates which are considered national
drop if year<1995  																// thresholds use only 2000+ data, but 1995-2000 used in 5 year lag calculations leading up to 2000
    
ren 	welfare_type welfare_type_b												// decode the variable defining income or consumption survey
decode  welfare_type_b, generate(welfare_type)
drop	welfare_type_b

replace gini = gini * 100
	

********************************************************************************
* Step 2: PREPARING PREFERRED ESTIMATES WHEN BOTH INCOME AND CONSUMPTION ARE AVAILABLE 
********************************************************************************

preserve
use "$ch2_input_raw/pfw_v11.dta", clear
keep if survey_coverage=="N" | inlist(code,"ARG") 
isid code rep_year datatype
keep if display_cp == 1															// The variable display_cp equals 1 for estimates that are prefered (should be displayed and 0 otherwise)
keep code rep_year datatype  	
rename code country_code
rename rep_year year
rename datatype ci
gen 	welfare_type = "consumption" if inlist(ci,"C","c")
replace welfare_type = "income" 	 if inlist(ci,"I","i")
drop ci
duplicates drop
tempfile pref
save 	`pref'
restore

merge m:1 country_code year welfare_type using `pref', keep(3) nogen


*** Create weight based on the number of surveys for each country and country-welfaretype (2000+ surveys only) ***
drop if year <2000
bysort country_code: 				gen weightall 	= 1/_N
bysort country_code welfare_type: 	gen weighttype 	= 1/_N


********************************************************************************
* Step 3: Expand to include all years 1995-2000 for each country, 
*			add income group, FCV, IDA groups and population estimates 
********************************************************************************	 

*** create a balanced sample of country-years *** 
preserve
	duplicates drop country_code, force
	keep country_name country_code
	expand 30
	gen year = 1995
	bys country_code: replace year = year[_n-1] + 1 if _n>=2
	tempfile yearly
	save 	`yearly', replace
restore

merge 1:1 country_code year using `yearly', nogen


*** Add WB classifications data ***
preserve 														
	use "$ch2_input_processed/CLASS_FY24.dta", clear
	* Correct country name
	replace economy = "Czechia" if economy == "Czech Republic"
	ren year_data year 
	ren code country_code
	keep country_code year incgroup* ida* fcv*
	tempfile classifications
	save 	`classifications'
restore
* Add data for 2023  
preserve
	use `classifications', clear
	keep if year == 2022
	replace year  = 2023 
	append using `classifications'
	sort country_code year
	tempfile classifications2
	save 	`classifications2'
restore 

	

merge m:1 country_code year using `classifications2', nogen keep(3)

sort country_code year
keep country_code country_name year welfare_time region* incgroup_current ida_current fcv_current comparable_spell survey_coverage welfare_type gini mean headcount


*** Add population data ***
preserve 																		
	use "$ch2_input_processed/sp_pop_totl.dta", clear
	keep if year >=1995
	keep country_code year value 
	ren value population
	tempfile pop
	save 	`pop'
restore

merge 1:1 country_code year using `pop', keep(1 3) nogen


********************************************************************************
* Step 4: Create the necessary variables and clean up data 
********************************************************************************

*** income and consumption ginis only ***
gen gini_consumption = gini if welfare_type=="consumption" 						
gen gini_income 	 = gini if welfare_type=="income" 							


*** Five-year forward ginis where no survey for up to 5 years ***
gen survey 		= !missing(welfare_type)
gen survey5 	= survey
gen surveyever 	= 1																// dataset only includes countries with atleast 1 survey, so all countries have at least 1 survey

forvalues i=1/4 {
	bys country_code (year): replace survey5 = 1 if survey5==0 & survey[_n-`i']==1
}


*** Time since last survey ***
encode country_name, gen(cntr)
xtset cntr year
bys cntr (year): gen  spell = sum( survey != L1.survey )
bys country_code spell (year): gen surveylag5 = _n
bys country_code spell (year): gen surveylagever = _n


replace surveylag5 		= 0 if survey5==0
replace surveylag5 		= . if survey==survey5
replace surveylagever 	= 0 if surveyever==0
replace surveylagever 	= . if survey==surveyever
drop cntr spell


*** Carry forward the inequality measure ***
foreach var of varlist gini {
	gen 	`var'5 = `var'
	bys country_code (year): replace `var'5 	= `var'[_n-surveylag5] if missing(`var'5)
	gen 	`var'ever = `var'
	bys country_code (year): replace `var'ever 	= `var'[_n-surveylagever] if missing(`var'ever)
}

*** Backcast values to get rid of 'missing data' ***
forvalues i = 1/40 {
	bys country_code (year): replace giniever = giniever[_n+`i'] if missing(giniever)
}

drop surveyever surveylagever survey survey5 surveylag5 


*** Income group, IDA, and FCV status for calculating classifications ***

replace ida_current = "Other" 	if ida_current == "Rest of the world"
replace fcv_current = "FCV" 	if fcv_current == "Yes"
replace fcv_current = "Not FCV" if fcv_current == "No"

replace incgroup_current= "HIC" 	if incgroup_current=="High income"
replace incgroup_current= "UMIC" 	if incgroup_current=="Upper middle income"
replace incgroup_current= "LMIC" 	if incgroup_current=="Lower middle income"
replace incgroup_current= "LIC" 	if incgroup_current=="Low income"


label var gini_consumption 	"Consumption-based ginis only"
label var gini_income 		"Income-based ginis only"
label var gini5 			"Forward-casted gini up to 5 years from current survey"
label var giniever 			"Forward-casted gini or backcasted gini from the first survey"



********************************************************************************
* Step 5: Save 
********************************************************************************


*** save the gini for the latest survey year only *** 
preserve
bysort country_code (country_code year)	: egen latest = max(year)  if !missing(gini)
keep if latest == year
drop latest
save "$ch2_input_processed/gini_intermediate_latest.dta", replace
restore

*** save data for all years post 2000 *** 
keep if inrange(year,2000,2022)													// 2022 is most comparable
save "$ch2_input_processed/gini_intermediate_all.dta", replace


********************************************************************************
exit