
/*******************************************************************************
	Download datasets for PSPR 24

	- Part 1: Country, region, income-level and fcs codes
	- Part 2: Population  

	Written by: Henry Stemmler
	Last date modified: 12 July 2024     
*******************************************************************************/

********************************************************************************
*   Part 1: Country, region, income-level and fcs codes                        
********************************************************************************

use "$ch3_data_raw/classification/CLASS.dta", clear
* Correct country name
replace economy = "Czechia" if economy == "Czech Republic"
* Save temporarily
tempfile cc
save `cc'
* Get region names from WDI
wbopendata, indicator(SP.POP.TOTL) projection year(2022) clear
* Keep relevant variables and observations only
keep 	countrycode region
drop 	if region == "" | region == "NA"
rename 	(countrycode region) (code region_wdi)
label 	var region_wdi "WDI region"
merge 	1:m code using "`cc'", nogen
label 	var economy "Country Name"
* Generate FCS variable fixed at 2022
gen 	fcv2022 = 1 if fcv_historical == "Yes" & year_data == 2022
bys 	code: egen fcv_2022 = max(fcv2022)
replace	fcv_2022 = 0 if fcv_2022 == .
label 	def yesno 1 "Yes" 0 "No"
label	values fcv_2022 yesno
label 	variable fcv_2022 "FCV status - 2022"
drop	fcv2022
* PIP Region names
gen 	region_pip_name="East Asia and Pacific"				if region_pip=="EAP"
replace region_pip_name="Europe and Central Asia" 			if region_pip=="ECA"
replace region_pip_name="Latin America and the Caribbean"	if region_pip=="LAC"
replace region_pip_name="Middle East and North Africa" 		if region_pip=="MNA"
replace region_pip_name="Rest of the world" 				if region_pip=="OHI"
replace region_pip_name="South Asia" 						if region_pip=="SAS"
replace region_pip_name="Sub-Saharan Africa"				if region_pip=="SSA"
* Add hyphens to income group names
replace incgroup_current = "Low-income" 		 			if incgroup_current == "Low income" 											// Numeric for order
replace incgroup_current = "Lower-middle-income"			if incgroup_current == "Lower middle income" 
replace incgroup_current = "Upper-middle-income"			if incgroup_current == "Upper middle income" 
replace incgroup_current = "High-income" 		 			if incgroup_current == "High income"
replace incgroup_historical = "Low-income" 					if incgroup_historical == "Low income" 											// Numeric for order
replace incgroup_historical = "Lower-middle-income" 		if incgroup_historical == "Lower middle income" 
replace incgroup_historical = "Upper-middle-income" 		if incgroup_historical == "Upper middle income" 
replace incgroup_historical = "High-income" 				if incgroup_historical == "High income"
gen ida_group_current = "IDA" if ida_current == "IDA" | ida_current == "Blend"
replace ida_group_current = "Other" if ida_group_current == ""
label var ida_group_current "IDA countries"																							// Put together Blend and IDA
* Add data for 2023 and 2024 
preserve 
keep if year_data == 2022
drop year_fiscal year_release
expand 2
bys code: replace year = 2023 if _n == 1
bys code: replace year = 2024 if _n == 2
tempfile f
save `f'
restore 
append using `f'
* Keep a version only for current year
keep 	if year_data == 2022
keep 	code economy region region_pip region_pip_name region_wdi region_SSA incgroup_current fcv_current fcv_2022 ida_group_current
save "$ch3_data_processed/countrycodes_current.dta", replace

* Small States
import excel "$ch3_data_raw/classification/FY25 small states.xlsx", clear firstrow
keep 	WB_Group_Code WB_Country_Code
rename	WB_Country_Code country_code
drop 	if WB_Group_Code == "LDC"
bys country_code (WB_Group_Code): gen num = _n
reshape wide WB_Group_Code, i(country_code) j(num)
replace WB_Group_Code2 = WB_Group_Code1 if WB_Group_Code2 == "" & WB_Group_Code1 == "SST" 
replace WB_Group_Code1 = "" if WB_Group_Code1 == "SST"
rename (WB_Group_Code1 WB_Group_Code2) (smallisland smallstate)
save "$ch3_data_processed/small_states.dta", replace


********************************************************************************
*   Part 2: Population    
*********************************
/* Code to retrieve data from PIP, uncomment to get raw data
global pipoptions = "server(dev) version(20240627_2017_01_02_PROD)"
* National population
pip tables, clear table(pop) ${pipoptions}
keep if data_level == "national"
rename value pop
keep country_code year pop
save "$ch3_data_processed/pop", replace
*/
********************************************************************************
*   Part 3: GDP    
*********************************
*wbopendata, indicator(NY.GDP.MKTP.KD) long clear
use "$ch3_data_raw/other/gdp_wdi", clear
keep 	countrycode region ny_gdp_mktp_kd year
rename (countrycode ny_gdp_mktp_kd) (code gdp)
drop 	if region == "" | region == "NA"
save "$ch3_data_processed/gdp", replace

