/*******************************************************************************

	Preparation of NGFS emission projection data
	***********************************
	- Part 1: Emission projections, global and country-level
	- Part 2: Temperature projection
	- Part 3: Electricity projection
	- Part 4: Primary energy projection
	
	Written by: Henry Stemmler
	Last date modified: 12 July 2024     
	
********/

********************************************************************************

********************************************************************************
* Part 1: Emissions
********************************************************************************
*** Global
* Import historical EDGAR data
use	"$ch3_data_processed/emissions_edgar_global.dta", clear
collapse (sum) emissions, by(year)
keep if year > 1999
rename emissions ind
gen num = 1
reshape wide ind, i(num) j(year)
drop num
tempfile glob
save 	`glob'
* Import NGFS data
import excel "$ch3_data_raw/ngfs_projections/ngfs_global_emissions.xlsx", clear firstrow
* Clean
rename *, lower																				// Lowercase
qui ds
loc last: word `c(k)' of `r(varlist)'										
foreach v of varlist f - `last' {															// Rename year variables
   local x : variable label `v'
   rename `v' ind`x'
}
drop unit region variable
* Shorten model names
replace model = "RM (median damage)" if model == "REMIND-MAgPIE 3.2-4.6 IntegratedPhysicalDamages (median)"
replace model = "RM (high damage)" 	 if model == "REMIND-MAgPIE 3.2-4.6 IntegratedPhysicalDamages (95th-high)"
replace model = "GCAM"				 if model == "GCAM 6.0 NGFS"
replace model = "MG"				 if model == "MESSAGEix-GLOBIOM 1.1-M-R12"
replace scenario = "Below 2C" if scenario =="Below 2?C"
drop ind2055 ind2060 ind2065 ind2070 ind2075 ind2080 ind2085 ind2090 ind2095 ind2100		// Keep up to 2050
* Make graph
keep if scenario == "Net Zero 2050" | scenario == "Current Policies" |  scenario == "Nationally Determined Contributions (NDCs)"
* Append historical
append using `glob'
* Generate annualized growth rate between 2020 - 2025
gen gr2025 = (ind2025/ind2020)^(1/5)
replace ind2022 = ind2020 * gr2025^2 if model != ""											// 2022 value based on constant growth between 2020 and 2025
replace ind2020 = . if model != ""
gen base = ind2022
foreach var of varlist ind*  {
	replace `var' = `var' / base
}
drop base gr2025
order ind*, alphabetic
order model scenario
replace model = "Historical" if model == ""
* Generate ranges
reshape long ind, i(model scenario) j(year)
bys scenario year: egen max_range = max(ind)
bys scenario year: egen min_range = min(ind)
save "$ch3_data_processed/edgar_ngfs_projections.dta", replace

*** Country
* Population projections
import excel "$ch3_data_raw/ngfs_projections/ngfs_rm_downscale_gdppop.xlsx", clear firstrow
* Clean
rename *, lower																				// Lowercase
qui ds
loc last: word `c(k)' of `r(varlist)'										
foreach v of varlist f - `last' {															// Rename year variables
   local x : variable label `v'
   rename `v' pop`x'
}
keep if model 	 == "Downscaling[REMIND-MAgPIE 3.2-4.6 IntegratedPhysicalDamages (median)]"
keep if scenario == "Current Policies"														// Population does not vary between scenarios
keep if variable == "Population"
drop unit variable model scenario
reshape long pop, i(region) j(year)
tempfile pop
save 	`pop'
* Import historical EDGAR data
use	"$ch3_data_processed/emissions_edgar_national.dta", clear
collapse (sum) emissions, by(code year)
keep if year > 1999
tempfile nat
save 	`nat'
import excel "$ch3_data_raw/ngfs_projections/ngfs_downscale_emissions.xlsx", clear firstrow
* Clean
rename *, lower																				// Lowercase
qui ds
loc last: word `c(k)' of `r(varlist)'										
foreach v of varlist f - `last' {															// Rename year variables
   local x : variable label `v'
   rename `v' ind`x'
}
drop unit variable
keep if model == "Downscaling[REMIND-MAgPIE 3.2-4.6 IntegratedPhysicalDamages (median)]"
reshape long ind, i(region model scenario) j(year)
* Merge population
merge m:1 region year using `pop'
rename  (ind region) (emissions code)
append using `nat'
* Adjust to 2020 emissions
gen em2020 = emissions if year == 2020 & model == ""
bys code year: egen em_2020 = mean(em2020)
gen adj_factor = emissions - em_2020
bys code scenario: egen adjust_factor = mean(adj_factor)
replace emissions = emissions - adjust_factor if model != ""							
drop if model == ""
drop em2020 em_2020 adj_factor adjust_factor

* Emissions per capita
gen emissions_pc = emissions/pop

sort code scenario year
save "$ch3_data_processed/edgar_ngfs_country_projections.dta", replace

********************************************************************************
* Part 2: Temperature
********************************************************************************

* Import historical temperature data from AR6
import delimited "$ch3_data_raw/other/gmst_changes_model_and_obs.csv", rowrange(38:) clear
rename (conventions v8) (year ind)
keep	year ind
drop	if ind == .
destring year, replace
keep if  year > 1999
gen num = 1
reshape wide ind, i(num) j(year)
drop num
tempfile glob
save 	`glob'

* Import NGFS data
import excel "$ch3_data_raw/ngfs_projections/ngfs_global_temp.xlsx", clear firstrow
* Clean
rename *, lower																				// Lowercase
qui ds
loc last: word `c(k)' of `r(varlist)'										
foreach v of varlist f - `last' {															// Rename year variables
   local x : variable label `v'
   rename `v' ind`x'
}

drop unit region variable
* Shorten model names
replace model = "RM (median damage)" if model == "REMIND-MAgPIE 3.2-4.6 IntegratedPhysicalDamages (median)"
replace model = "RM (high damage)" if model == "REMIND-MAgPIE 3.2-4.6 IntegratedPhysicalDamages (95th-high)"
replace model = "GCAM" if model == "GCAM 6.0 NGFS"
replace model = "MG" if model == "MESSAGEix-GLOBIOM 1.1-M-R12"
replace scenario = "Below 2C" if scenario =="Below 2?C"
drop ind2055 ind2060 ind2065 ind2070 ind2075 ind2080 ind2085 ind2090 ind2095 ind2100		// Keep up to 2050
* Make graph
keep if scenario == "Net Zero 2050" | scenario == "Current Policies" |  scenario == "Nationally Determined Contributions (NDCs)"
append using `glob'
sum ind2020 if model == ""
gen share = r(sum) / ind2020																// Conversion factor to get everything to the same rate as historical 2020
foreach var of varlist ind*  {
	replace `var' = `var' * share
}
drop share
order ind*, alphabetic
order model scenario
replace model = "Historical" if model == ""

* Generate ranges
reshape long ind, i(model scenario) j(year)
bys scenario year: egen max_range = max(ind)
bys scenario year: egen min_range = min(ind)

save "$ch3_data_processed/ngfs_temp_projections.dta", replace


********************************************************************************
* Part 3: Electricity 
********************************************************************************
import excel "$ch3_data_raw/ngfs_projections/ngfs_downscale_electricity.xlsx", clear firstrow
* Clean
rename *, lower																		// Lowercase
qui ds
loc last: word `c(k)' of `r(varlist)'										
foreach v of varlist f - `last' {													// Rename year variables
   local x : variable label `v'
   rename `v' ind`x'
}
drop model scenario
drop ind2055 ind2060 ind2065 ind2070 ind2075 ind2080 ind2085 ind2090 ind2095 ind2100		// Keep up to 2050
rename region code
* Merge with WB classifications 
merge 	m:1 code using "$ch3_data_processed/countrycodes_current.dta", keep(3) nogen
replace variable = subinstr(variable,"Secondary Energy|Electricity|", "", .)		// Remove part of string
replace variable = "Oil, Gas, Coal" if inlist(variable, "Oil", "Gas", "Coal")
replace variable = "Renewable" if inlist(variable, "Geothermal", "Wind", "Solar", ///
				"Hydro", "Biomass")
drop if variable == "Non-Biomass Renewables"										// Non-Biomass Renewables should be included in the rest
save "$ch3_data_processed/ngfs_electricity_projections.dta", replace


********************************************************************************
* Part 4: Primary energy 
********************************************************************************
import excel "$ch3_data_raw/ngfs_projections/ngfs_downscale_primenergy.xlsx", clear firstrow
* Clean
rename *, lower																		// Lowercase
qui ds
loc last: word `c(k)' of `r(varlist)'										
foreach v of varlist f - `last' {													// Rename year variables
   local x : variable label `v'
   rename `v' ind`x'
}
drop model scenario
drop ind2055 ind2060 ind2065 ind2070 ind2075 ind2080 ind2085 ind2090 ind2095 ind2100		// Keep up to 2050
rename region code
* Merge with WB classifications 
merge 	m:1 code using "$ch3_data_processed/countrycodes_current.dta", keep(3) nogen
replace variable = subinstr(variable,"Primary Energy|", "", .)						// Remove part of string
drop if inlist(variable, "Oil", "Gas", "Coal")										// Fossil aggregate
drop if inlist(variable, "Non-Biomass Renewables")									// Fossil aggregate
replace variable = "Renewable" if variable != "Fossil" & variable != "Nuclear"
collapse (sum) ind*, by(code variable incgroup_current)
save "$ch3_data_processed/ngfs_primenergy_projections.dta", replace
