/*******************************************************************************
	Figure 1: The number of household budget surveys has increased in all regions 

	Written by: Samuel Kofi Tetteh Baah (stettehbaah@worldbank.org)
	Last date modified: 6 August 2024     
*******************************************************************************/

// Determine country types 
use "$ch4_data_processed/countrycodes_yearly.dta", clear 
keep code ida_current
duplicates drop 
replace ida_current = "IDA" if ida_current=="Blend"
keep if ida_current=="IDA" 

duplicates drop  
rename ida_current type
tempfile 1 
save `1'

use "$ch4_data_processed/countrycodes_yearly.dta", clear 
keep code fcv_current
keep if fcv_current == "Yes"
rename  fcv_current type 
replace type = "FCV" 
duplicates drop 


tempfile 2 
save `2'


use "$ch4_data_processed/countrycodes_yearly.dta", clear 
keep code region_pip  
duplicates drop
rename region_pip type  

tempfile 3 
save `3'


use "$ch4_data_processed/countrycodes_yearly.dta", clear  
keep code incgroup_current
duplicates drop  
rename incgroup_current type

tempfile 4 
save `4'


use "$ch4_data_processed/countrycodes_yearly.dta", clear  
keep code 
duplicates drop
gen type = "WLD"

tempfile 5
save `5'

use "$ch4_data_processed/countrycodes_current.dta", clear
keep code small_state
keep if small_state == "Yes"
rename  small_state type 
replace type = "SST" 
duplicates drop
 
tempfile 6 
save `6'

// Combine data sets 
use `1', clear 
append using `2'
append using `3'
append using `4'
append using `5'
append using `6'
rename code country_code 
replace type = "HIC" if type=="High-income"
replace type = "UMIC" if type=="Upper-middle-income"
replace type = "LMIC" if type=="Lower-middle-income"
replace type = "LIC" if type=="Low-income"
egen typeid = group(type)
sum typeid
drop typeid

save "$ch4_data_processed/country_groups_current.dta", replace 

// Load population data 
use "$ch4_data_processed/pop_reportinglevel.dta", clear 
egen pop_nat_ = mean(pop) if reporting_level=="national",by(country_code year)
egen pop_nat = mean(pop_nat_),by(country_code year)
drop pop_nat_
gen pop_wgt = pop/pop_nat
tempfile pop 
save `pop'
save "$ch4_data_processed/pop_group_data.dta", replace

// Load survey data 
use "$ch4_data_processed/poverty_country_survey.dta", clear 
keep country_code year welfare_time welfare_type reporting_level
drop if inlist(reporting_level,"rural","urban") & inlist(country_code,"CHN","IND","IDN")
duplicates drop 
gen double welfare_time_ceil = ceil(welfare_time)
keep country_code welfare_type welfare_time_ceil reporting_level
rename welfare_time_ceil year 

bysort country_code year reporting_level: gen dup=_N 
drop if dup==2 
drop dup welfare_type

isid country_code year 
drop if year<1981
keep if year<=2024
gen coverage = 1
save "$ch4_data_processed/surveys_1981_2024.dta", replace 

capture erase "$ch4_data_processed/coverage_group_results.dta"
drop _all
save "$ch4_data_processed/coverage_group_results.dta", empty


use  "$ch4_data_processed/country_groups_current.dta", clear
levelsof type, local(groups)
foreach g of local groups {
preserve
keep if type=="`g'"	

tempfile type 
save `type'


forvalues i = 1998/2023{
use "$ch4_data_processed/surveys_1981_2024.dta", clear
keep if year<=`i'
merge 1:1 country_code year reporting_level using "$ch4_data_processed/pop_group_data.dta", nogen
merge m:1 country_code using `type', nogen keep(3 2)
sort country_code year reporting_level
drop if inlist(reporting_level,"rural","urban") & coverage==.
sort country_code year reporting
egen double drop_year = total(coverage),by(country_code year)
drop if coverage==. & drop_year==1  // if there is urban/rural survey drop national observation
isid country_code year // there should be country-year observations
drop drop_year
replace coverage = 0 if coverage==.
drop if year<1981
gen coverage_pop = coverage*pop

egen cc = group(country_code)
sum cc 
gen countries = `r(max)'
collapse (sum) coverage (mean) countries,by(type) 
gen survey_pc = coverage/countries 
gen double year= `i'
drop coverage countries 
append using "$ch4_data_processed/coverage_group_results.dta"
sleep 500
save "$ch4_data_processed/coverage_group_results.dta", replace
sleep 500 
}
restore
}

// Make charts
use "$ch4_data_processed/coverage_group_results.dta", clear 
encode type, gen(type_)
rename survey_pc y
reshape wide y, i(type year) j(type_)
rename year time
collapse y*,by(time)


foreach var in EAP ECA FCV HIC IDA LAC LIC LMIC MNA OHI SAS SSA SST UMIC WLD{ 
forvalues j = 1/15{
	capture rename y`j' y_`var'
} 
}

gen type = "type"
rename time year 

// Make chart
keep if year<=2022
line y_LIC y_LMIC y_UMIC y_HIC y_FCV y_IDA y_SST y_WLD year, lwidth(medium medium medium medium medium medium medium thick) lpattern(solid solid solid solid solid solid dash solid) sort xlab(1998(4)2022) ///
	ytitle("Cumulative number of surveys")   ///
	lcolor("255 213 137" "255 129 51" "153 245 255" "14 100 169" "163 163 163" "237 52 46" "71 71 72" "71 71 72") ///
 legend(order(1 "Low-income" ///
	2 "Lower-middle-income" 3 "Upper-middle-income" ///
	4 "High-income" 5 "FCS" 6 "IDA" 7 "Small states" 8 "World") symxsize(*.6) symysize(*.8) ///
	position(6) col(2) size(medium)) xtitle("") ysize(7) xsize(8)
	
	
graph export "$ch4_output/ch4_fig1_surveys.eps", as(eps) replace
graph export "$ch4_output/ch4_fig1_surveys.png", as(png) replace 	

export excel "$ch4_output/chapter4.xlsx", sheet("fig1", modify) firstrow(varlabels)


erase "$ch4_data_processed/country_groups_current.dta"
erase "$ch4_data_processed/pop_group_data.dta"
erase  "$ch4_data_processed/surveys_1981_2024.dta"
