/*******************************************************************************
	Figure 2: Less than One-Half of the Countries in the World Have Data Available for Global Monitoring of Poverty in 2020 or later

	Written by: Samuel Kofi Tetteh Baah (stettehbaah@worldbank.org)
	Last date modified: 9 August 2024     
************************************************************************/



// Prepare data on classification of countries

// Small states 
use "$ch4_data_processed/countrycodes_current.dta", clear
keep code small_state
keep if small_state == "Yes"
rename  small_state type 
replace type = "SST" 
gen group = "SST"

tempfile sst 
save `sst'

use "$ch4_data_processed/countrycodes_yearly.dta", clear 
keep code year_data 
rename year_data year
merge m:1 code using `sst', nogen keep(match)
rename code country_code 
tempfile sst 
save `sst'
 

// PIP regions
use "$ch4_data_processed/countrycodes_yearly.dta", clear 
keep code year_data region_pip
rename year_data year
rename region_pip group 
gen type = "PIP"
duplicates drop 
rename code country_code 
count 

tempfile pip_group
save `pip_group'


// Income group
use "$ch4_data_processed/countrycodes_yearly.dta", clear 
keep code year_data incgroup_historical
rename year_data year
rename incgroup_historical group 
gen type = "Income"
duplicates drop 
rename code country_code 
count 

tempfile inc_group
save `inc_group'

// FCV
use "$ch4_data_processed/countrycodes_yearly.dta", clear 
keep code year_data fcv_historical
rename year_data year
gen group = "FCV" if fcv_historical=="Yes"
replace group = "Non-FCV" if fcv_historical=="No"
drop fcv_historical
drop if group==""
gen type = "FCV"
rename code country_code 
count 

tempfile fcv_group
save `fcv_group'


// IDA 
use "$ch4_data_processed/countrycodes_yearly.dta", clear
keep code year_data ida_historical
rename year_data year
gen group = "IDA" if inlist(ida_historical,"IDA","Blend")
replace group = "IBRD" if inlist(ida_historical,"IBRD")
replace group = "Rest of the World" if inlist(ida_historical,"Rest of the world")
drop ida_historical
drop if group==""
gen type = "IDA"
rename code country_code 
count 

tempfile ida_group
save `ida_group'

// Combine groups 
use `ida_group', clear 
append using `fcv_group'
append using `inc_group'
append using `pip_group'
append using `sst'

tempfile country_groups 
save `country_groups'






/////////////// Prepare country-level coverage data  ///////////////// 
drop _all
capture erase "$ch4_data_processed/coverage_pppr_2024_covid.dta"
save "$ch4_data_processed/coverage_pppr_2024_covid.dta", empty

// Load PIP data
use "$ch4_data_processed/poverty_country_survey.dta", clear 
keep welfare_time country_code 
drop if welfare_time<2019.5
duplicates drop 
gen j = _n
count 
forvalues i = 1/`r(N)'{
preserve 	
	keep if j==`i' 
	gen threshold_lo = welfare_time - 3
	gen threshold_hi = welfare_time + 3
	expand 4
	gen year = 2020 in 1
	replace year = year[_n-1] + 1 if year==.
	drop if year>threshold_hi | year<threshold_lo
	keep country_code year

	append using "$ch4_data_processed/coverage_pppr_2024_covid.dta"
	duplicates drop 
	sort country_code year 
	
	save "$ch4_data_processed/coverage_pppr_2024_covid.dta", replace
restore 
} 


//////////////////////////////   Combine data  //////////////////////////////  
use "$ch4_data_processed/coverage_pppr_2024_covid.dta", clear
gen reporting_level = "national"
replace reporting_level = "urban" if country_code=="ARG" 
gen coverage = 1
save "$ch4_data_processed/coverage_country_level_pppr_2024.dta", replace



///////////////  Combine coverage data with population numbers /////////////// 
use "$ch4_data_processed/pop_reportinglevel.dta", clear

merge 1:1 country_code year reporting_level using "$ch4_data_processed/coverage_country_level_pppr_2024.dta"
keep if _merge==3 | _merge==1
drop _merge 
drop if year<1990
sort country_code year reporting_level
keep if reporting_level=="national" | inlist(country_code,"ARG")
drop if reporting_level=="national" & inlist(country_code,"ARG")
sort country_code year reporting_level

// Check that Argentina and Suriname have only rural and urban sectors for all years since 1990
bysort country_code year (reporting_level): gen j = _n 
egen count_year = group(year)
sum count_year  // the total count of years

isid country_code year reporting_level 

egen count_country = group(country_code)
sum count_country // there should be 218 countries 
drop j count_*
replace coverage = 0 if coverage==.
save "$ch4_data_processed/coverage_country_level_pppr_2024_clean.dta", replace



////////////////////////////// Coverage without population weighting  //////////////////////////


// Load coverage data 
use "$ch4_data_processed/coverage_country_level_pppr_2024_clean.dta", clear
drop if year>2022
drop if year<2020
expand 6
bysort country_code year reporting_level: gen j = _n 
gen type = "Income" if j==1 
replace type = "FCV" if j==2
replace type = "IDA" if j==3
replace type = "PIP" if j==4
replace type = "WLD" if j==5
replace type = "SST" if j==6
merge m:1 country_code year type using `country_groups'
drop if pop==.
replace group = "WLD" if type=="WLD"
sum year if _merge==1
drop if group == ""
replace group = type if type=="FCV" & group=="Yes"
replace group = "Non FCV" if type=="FCV" & group=="No"
count if type == "IDA" & year==1998

// Simply select the latest year 
keep if year==2022
isid country_code reporting_level group
bysort country_code: gen reporting_level_id = _N 
drop reporting_level_id
egen pop_nat_ = sum(pop) if group=="WLD",by(country_code)
egen pop_nat = mean(pop_nat_),by(country_code)
drop pop_nat_
rename pop pop_
gen pop = pop_/pop_nat  // Assign each country a weight of 1
egen pop_grp = sum(pop),by(group year type)

gen cov_pop_grp = coverage*pop
egen cov_pop_grp_all = sum(cov_pop_grp),by(group year type)

gen coverage_result = 100*cov_pop_grp_all/ pop_grp
keep year coverage_result group type
duplicates drop 

sort coverage_result

drop if inlist(group,"Rest of the world","IBRD")
sort group 

preserve 
keep if type=="Income"
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 hic 
rename cov2 lic 
rename cov3 lmic
rename cov4 umic 

tempfile df1 
save `df1'
restore

preserve 
keep if type=="PIP"
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 eap 
rename cov2 eca 
rename cov3 lac
rename cov4 mna
rename cov5 ohi 
rename cov6 sas 
rename cov7 ssa

tempfile df2 
save `df2'
restore


preserve 
drop if year<1998
drop if inlist(type,"PIP","Income","WLD","SST")
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year) 
rename cov1 fcv  
rename cov2 ida 
rename cov3 nfcv 

tempfile df3 
save `df3'
restore


preserve 
keep if inlist(type,"WLD")
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 wld  

tempfile df4 
save `df4'
restore


preserve 
keep if inlist(type,"SST")
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 sst  

tempfile df5 
save `df5'
restore

// Combine data sets
use `df1', clear 
append using `df2'
append using `df3'
append using `df4'
append using `df5'

collapse lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv sst wld,by(year)
sort year 

save "$ch4_data_processed/coverage_pppr_2024_results_groups_countries.dta", replace


////////////////////////////// Coverage with population weighting  //////////////////////////

// Load coverage data 
use "$ch4_data_processed/coverage_country_level_pppr_2024_clean.dta", clear
drop if year>2022
drop if year<2020
expand 6
bysort country_code year reporting_level: gen j = _n 
gen type = "Income" if j==1 
replace type = "FCV" if j==2
replace type = "IDA" if j==3
replace type = "PIP" if j==4
replace type = "WLD" if j==5
replace type = "SST" if j==6
merge m:1 country_code year type using `country_groups'
drop if pop==.
replace group = "IDA" if group=="Blend"
replace group = "WLD" if type=="WLD"
sum year if _merge==1
drop if group == ""
replace group = type if type=="FCV" & group=="Yes"
replace group = "Non FCV" if type=="FCV" & group=="No"
count if type == "IDA" & year==1998

// Simply select the latest year 
keep if year==2022


isid country_code reporting_level group

egen pop_grp = sum(pop),by(group year type)

gen cov_pop_grp = coverage*pop
egen cov_pop_grp_all = sum(cov_pop_grp),by(group year type)

gen coverage_result = 100*cov_pop_grp_all/ pop_grp
keep year coverage_result group type
duplicates drop 

sort coverage_result

drop if inlist(group,"Rest of the world","IBRD")

preserve 
keep if type=="Income"
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 hic 
rename cov2 lic 
rename cov3 lmic
rename cov4 umic 

tempfile df1 
save `df1'
restore

preserve 
keep if type=="PIP"
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 eap 
rename cov2 eca 
rename cov3 lac
rename cov4 mna
rename cov5 ohi 
rename cov6 sas 
rename cov7 ssa

tempfile df2 
save `df2'
restore


preserve 
drop if year<1998
drop if inlist(type,"PIP","Income","WLD","SST")
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year) 
rename cov1 fcv  
rename cov2 ida 
rename cov3 nfcv 

tempfile df3 
save `df3'
restore


preserve 
keep if inlist(type,"WLD")
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 wld  

tempfile df4 
save `df4'
restore


preserve 
keep if inlist(type,"SST")
isid group year 
bysort year (group): gen j = _n
rename coverage_result cov 
reshape wide cov, i(group year) j(j)
collapse cov*, by(year)
rename cov1 sst  

tempfile df5 
save `df5'
restore

// Combine data sets
use `df1', clear 
append using `df2'
append using `df3'
append using `df4'
append using `df5'

collapse lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv sst wld,by(year)
sort year
save "$ch4_data_processed/coverage_pppr_2024_results_groups_countries_pop.dta", replace




////////////////////////////// Make chart  //////////////////////////

* Without population weighting
use "$ch4_data_processed/coverage_pppr_2024_results_groups_countries.dta", clear
local j = 1
foreach var in lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv sst wld{
	rename `var' y`j'
	local j = `j'+1
}

reshape long y, i(year) j(j)

gen type = ""
local j = 1
foreach var in lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv sst wld{
replace type = "`var'" if j==`j'
	local j = `j'+1
}

replace type = upper(type)
drop if type=="BLD" | type == "NFCV"
drop j
rename y coverage 
rename type group  
rename coverage pppr 

gen type = "Region" if !(inlist(group,"IDA","FCV") | inlist(group,"LIC","UMIC","HIC","LMIC")) 
replace type = "Income" if inlist(group,"LIC","UMIC","HIC","LMIC")
replace type = "Other" if (inlist(group,"IDA","FCV","SST")) 
replace group = "Low-income" if group=="LIC"
replace group = "Lower-middle-income" if group=="LMIC"
replace group = "Upper-middle-income" if group=="UMIC"
replace group = "High-income" if group=="HIC"
replace group= "East Asia & Pacific" if group=="EAP"	
replace group= "Europe & Central Asia" if group=="ECA"			
replace group= "Latin America & Caribbean" if group=="LAC"	
replace group= "Middle East & North Africa" if group=="MNA"	
replace group= "Rest of the World" if group=="OHI"
replace group= "South Asia" if group=="SAS"
replace group= "Sub-Saharan Africa" if group=="SSA"
replace group = "FCS" if group=="FCV"
replace group = "IDA" if group=="IDA"
replace group = "Small states" if group=="SST"
replace group = "World" if group=="WLD"
drop if inlist(group,"Non-FCV")

tempfile 1 
save `1'


* With population weighting
use "$ch4_data_processed/coverage_pppr_2024_results_groups_countries_pop.dta", clear
local j = 1
foreach var in lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv sst wld{
	rename `var' y`j'
	local j = `j'+1
}

reshape long y, i(year) j(j)

gen type = ""
local j = 1
foreach var in lic hic lmic umic eap eca lac mna ohi sas ssa fcv ida nfcv sst wld{
replace type = "`var'" if j==`j'
	local j = `j'+1
}

replace type = upper(type)
drop if type=="BLD" | type == "NFCV"
drop j
rename y coverage 
rename type group  
rename coverage pppr_pop 

gen type = "Region" if !(inlist(group,"IDA","FCV") | inlist(group,"LIC","UMIC","HIC","LMIC")) 
replace type = "Income" if inlist(group,"LIC","UMIC","HIC","LMIC")
replace type = "Other" if (inlist(group,"IDA","FCV","SST")) 
replace group = "Low-income" if group=="LIC"
replace group = "Lower-middle-income" if group=="LMIC"
replace group = "Upper-middle-income" if group=="UMIC"
replace group = "High-income" if group=="HIC"
replace group= "East Asia & Pacific" if group=="EAP"	
replace group= "Europe & Central Asia" if group=="ECA"			
replace group= "Latin America & Caribbean" if group=="LAC"	
replace group= "Middle East & North Africa" if group=="MNA"	
replace group= "Rest of the World" if group=="OHI"
replace group= "South Asia" if group=="SAS"
replace group= "Sub-Saharan Africa" if group=="SSA"
replace group = "FCS" if group=="FCV"
replace group = "IDA" if group=="IDA"
replace group = "World" if group=="WLD"
replace group = "Small states" if group=="SST"
drop if inlist(group,"Non-FCV")

merge 1:1 group using `1', nogen

gen sorting_var1 = 1 if group=="East Asia & Pacific"
replace sorting_var1 = 2 if group=="Europe & Central Asia"
replace sorting_var1 = 3 if group=="Latin America & Caribbean"
replace sorting_var1 = 4 if group=="Middle East & North Africa" 
replace sorting_var1 = 5 if group=="South Asia"
replace sorting_var1 = 6 if group=="Sub-Saharan Africa"
replace sorting_var1 = 7 if group=="Rest of the World"
replace sorting_var1 = 8 if group=="World"
replace sorting_var1 = 9 if group=="Low-income"
replace sorting_var1 = 10 if group=="Lower-middle-income"
replace sorting_var1 = 11 if group=="Upper-middle-income"
replace sorting_var1 = 12 if group=="High-income"
replace sorting_var1 = 13 if group=="FCS"
replace sorting_var1 = 14 if group=="IDA"
replace sorting_var1 = 15 if group=="Small states"


gen sorting_var3 = 1 if type=="Region"
replace sorting_var3 = 2 if type=="Income"
replace sorting_var3 = 3 if type=="Other"

rename pppr y1 
rename pppr_pop y2 

reshape long y, i(group type sorting_var1) j(j)
egen sorting_var2 = group(sorting_var1 j)
reshape wide y, i(group type sorting_var1 j) j(sorting_var2)

sort sorting_var1 j

graph hbar y1-y30, over(group, sort(sorting_var1)) over(sorting_var3) nofill ytitle("Percent (%)")  ///
	bar(1, lwidth(medium) lpattern(solid) lcolor(rgb("237 52 46"))  fcolor(rgb("255 255 255"))) ///
	bar(2, lwidth(medium) lpattern(solid) lcolor(rgb("237 52 46"))  fcolor(rgb("237 52 46"))) ///
	bar(3, lwidth(medium) lpattern(solid) lcolor(rgb("163 163 163"))  fcolor(rgb("255 255 255"))) ///
	bar(4, lwidth(medium) lpattern(solid) lcolor(rgb("163 163 163"))  fcolor(rgb("163 163 163"))) ///
	bar(5, lwidth(medium) lpattern(solid) lcolor(rgb("58 190 203"))  fcolor(rgb("255 255 255"))) ///
	bar(6, lwidth(medium) lpattern(solid) lcolor(rgb("58 190 203"))  fcolor(rgb("58 190 203"))) ///
	bar(7, lwidth(medium) lpattern(solid) lcolor(rgb("14 100 169"))  fcolor(rgb("255 255 255"))) ///
	bar(8, lwidth(medium) lpattern(solid) lcolor(rgb("14 100 169"))  fcolor(rgb("14 100 169"))) ///
	bar(9, lwidth(medium) lpattern(solid) lcolor(rgb("255 129 51"))  fcolor(rgb("255 255 255"))) ///
	bar(10, lwidth(medium) lpattern(solid) lcolor(rgb("255 129 51"))  fcolor(rgb("255 129 51"))) ///
	bar(11, lwidth(medium) lpattern(solid) lcolor(rgb("255 213 137"))  fcolor(rgb("255 255 255"))) ///
	bar(12, lwidth(medium) lpattern(solid) lcolor(rgb("255 213 137"))  fcolor(rgb("255 213 137"))) ///
	bar(13, lwidth(medium) lpattern(solid) lcolor(rgb("153 245 255"))  fcolor(rgb("255 255 255"))) ///
	bar(14, lwidth(medium) lpattern(solid) lcolor(rgb("153 245 255"))  fcolor(rgb("153 245 255"))) ///
	bar(15, lwidth(medium) lpattern(solid) lcolor(rgb("71 71 72"))  fcolor(rgb("255 255 255"))) ///
	bar(16, lwidth(medium) lpattern(solid) lcolor(rgb("71 71 72"))  fcolor(rgb("71 71 72"))) ///
	bar(17, lwidth(medium) lpattern(solid) lcolor(rgb("255 213 137"))  fcolor(rgb("255 255 255"))) ///
	bar(18, lwidth(medium) lpattern(solid) lcolor(rgb("255 213 137"))  fcolor(rgb("255 213 137"))) ///
	bar(19, lwidth(medium) lpattern(solid) lcolor(rgb("255 129 51"))  fcolor(rgb("255 255 255"))) ///
	bar(20, lwidth(medium) lpattern(solid) lcolor(rgb("255 129 51"))  fcolor(rgb("255 129 51"))) ///
	bar(21, lwidth(medium) lpattern(solid) lcolor(rgb("153 245 255"))  fcolor(rgb("255 255 255"))) ///
	bar(22, lwidth(medium) lpattern(solid) lcolor(rgb("153 245 255"))  fcolor(rgb("153 245 255"))) ///
	bar(23, lwidth(medium) lpattern(solid) lcolor(rgb("14 100 169"))  fcolor(rgb("255 255 255"))) ///
	bar(24, lwidth(medium) lpattern(solid) lcolor(rgb("14 100 169"))  fcolor(rgb("14 100 169"))) ///
	bar(25, lwidth(medium) lpattern(solid) lcolor(rgb("163 163 163"))  fcolor(rgb("255 255 255"))) ///
	bar(26, lwidth(medium) lpattern(solid) lcolor(rgb("163 163 163"))  fcolor(rgb("163 163 163"))) ///
	bar(27, lwidth(medium) lpattern(solid) lcolor(rgb("237 52 46"))  fcolor(rgb("255 255 255"))) ///
	bar(28, lwidth(medium) lpattern(solid) lcolor(rgb("237 52 46"))  fcolor(rgb("237 52 46"))) ///
	bar(29, lwidth(medium) lpattern(solid) lcolor(rgb("71 71 72"))  fcolor(rgb("255 255 255"))) ///
	bar(30, lwidth(medium) lpattern(solid) lcolor(rgb("71 71 72"))  fcolor(rgb("71 71 72"))) ///
	ylab(0(25)100) yline(50) legend(position(6)) ///
	legend(order(15 "Share of countries" 16 "Share of population") ///
		row(1) symxsize(*.5) symysize(*.8) size(medsmall)) ysize(8) xsize(10)

// Delete sorting_var3 (1, 2, 3) in the Graph editor, and save as .eps and .pdf files 
/* --> the above step should be done manually */ 

graph export "$ch4_output/ch4_fig2_coverage.eps", as(eps) replace
graph export "$ch4_output/ch4_fig2_coverage.png", as(png) replace 	

export excel "$ch4_output/chapter4.xlsx", sheet("fig2", modify) firstrow(varlabels)

