
/*******************************************************************************
	Import exposure and vulnerability data
	
	Written by: Henry Stemmler
	Last modified 28 March 2024
*******************************************************************************/

********************************************************************************
*   Import Excel File for 2021   
********************************************************************************
use "$overview_data_raw/vulnerability/Exp_vul_rai_2021_raw.dta", clear
* Remove conflicted areas 
drop if geo_code2_new=="RUS_2015_GADM1_UKR_UKR.20_1"
drop if geo_code2_new=="RUS_2015_GADM1_UKR_UKR.4_1"
drop if geo_code2_new=="MAR_2015_DKx_268_1"
drop if geo_code2_new=="MAR_2015_DKx_268_2"
* Keep only 100-year return-period
keep if scenario == "RP100*"
* Exposed and vulnerable (both HH vulnerable and spatial vulnerable)
foreach var of varlist multvul_215 all2vul_215 multvul_685 all2vul_685 {
	gen double `var'_exp = exprai_ + (exp_ - exprai_)*`var'	
}
* Exposed and vulnerable (both HH vulnerable and spatial vulnerable)
foreach var of varlist poor215_ln poor685_ln dep_educ_com dep_infra_elec dep_infra_impw dep_sp dep_fin {
	gen double `var'_exp =  (exp_) * `var'	
}
* Aggregate to country-hazard level
collapse (sum) multvul_215_exp totalpop exp_ poor215_ln_exp poor685_ln_exp dep_educ_com_exp dep_infra_elec_exp dep_infra_impw_exp dep_sp_exp dep_fin_exp exprai_, by(hazard code)
* Clean up
rename multvul_215_exp risk_
rename poor215_ln_exp poor215_
rename poor685_ln_exp poor685_
reshape wide exp_ risk_ poor215_ poor685_ exprai_ dep_*, i(code) j(hazard) string
* Merge in other country information
merge 1:1 code using "$overview_data_processed/countrycodes_current.dta"
assert 	_merge != 1
keep if _merge == 3 
drop 	_merge
save "$overview_data_processed/exposure_vulnerability_2021.dta", replace

********************************************************************************
*   Import Excel Files for 2010 and 2019 to compute changes   
********************************************************************************

foreach y in 2010 2019 {
	use "$overview_data_raw/vulnerability/pov_vul6a_climate_data_`y'.dta", clear
	* Generate poor and exposed 
	gen 	expvul = 20 if  exp == "exp_any" & vul == "poorln_"	& line == 215	// Population exposed and poor at 2.15
	replace expvul = 21 if  exp == "exp_any" & vul == "poorln_"	& line == 685	// Population exposed and poor at 6.85
	* Keep only 2.15 * 1.5 line	(income vulnerability)
	drop if line != 322 & expvul == .
	* Set vulnerability to missing for countries with outdated baseyear (with exceptions) [based on Minh's code, but does not use type. In their code only based on baseline after 2014, but this drops most vulnerability for 2010 ]
	local vul multvul_ poorln_ dep_educ_com dep_fin dep_infra_elec dep_infra_impw dep_sp
	foreach v of local vul {
		replace nvul = . if vul == "`v'" & (type == 0 | `y' - baseyear > 5) & code != "IND" & code != "CHN" & line == 322 // & code != "CHL" & code != "MUS"
	}
	* Identify exposed 
	replace	expvul = 1 if  vul == "exp"	& exp == "exp_any"						// Population exposed to any shock
	replace expvul = 2 if  vul == "exp" & exp == "exp_cyclone"					// Population exposed to cyclone
	replace expvul = 3 if  vul == "exp" & exp == "exp_drought"					// Population exposed to drought
	replace expvul = 4 if  vul == "exp" & exp == "exp_flood"					// Population exposed to flood
	replace expvul = 5 if  vul == "exp" & exp == "exp_heat"						// Population exposed to heat	
	* Identify vulnerable 	
	replace expvul = 10 if  exp == "exp_any" & vul == "multvul_"				// Population exposed and vulnerable in any dimension
	replace expvul = 11 if  exp == "exp_any" & vul == "poorln_"	& line == 322	// Population exposed and vulnerable in income
	replace expvul = 12 if  exp == "exp_any" & vul == "dep_educ_com"			// Population exposed and vulnerable in education
	replace expvul = 13 if  exp == "exp_any" & vul == "dep_fin"					// Population exposed and vulnerable in financial inclusion
	replace expvul = 14 if  exp == "exp_any" & vul == "dep_infra_elec"			// Population exposed and vulnerable in electricity 
	replace expvul = 15 if  exp == "exp_any" & vul == "dep_infra_impw"			// Population exposed and vulnerable in water 
	replace expvul = 16 if  exp == "exp_any" & vul == "dep_sp"					// Population exposed and vulnerable in social protection

	* get population 
	gen pop = nvul	   if  vul == "tot" 
	bys code: egen pop_ev = mean(pop)
	drop if expvul == .
	keep code expvul regionname incomelevelname nvul pop_ev baseyear
	* Reshape and rename
	reshape wide nvul, i(code) j(expvul)
	rename nvul1 pop_exp_any
	rename nvul2 pop_exp_cyclone
	rename nvul3 pop_exp_drought
	rename nvul4 pop_exp_flood
	rename nvul5 pop_exp_heat
	rename nvul10 vuln_any
	rename nvul11 vuln_inc
	rename nvul12 vuln_educ
	rename nvul13 vuln_fin
	rename nvul14 vuln_elec
	rename nvul15 vuln_wat
	rename nvul16 vuln_soc
	rename nvul20 poor_exp_any_215
	rename nvul21 poor_exp_any_685	
	gen pop_withvuldat	= pop_ev if vuln_any != . 						// Population variable for those with vulnerability
	label var pop_withvuldat "Population of countries with vulnerability"
	label var pop_ev 		 "Population of countries with exposure"
	* Merge in other country information
	merge 1:1 code using "$overview_data_processed/countrycodes_current.dta"
	assert 	_merge != 1
	keep if _merge == 3 
	drop 	_merge

	* Save year
	gen year = `y'
	tempfile y`y'
	save `y`y'', replace
}
clear
append using `y2010' `y2019'
drop region_pip region_pip_name region region_SSA incgroup_current fcv_current fcv_2022 ida_group_current
* Identify countries with vulnerability data in both years
keep if pop_withvuldat != .
bys code: keep if  _N == 2
* Check for coverage and add ida
rename code country_code
merge 1:1 country_code year using "$overview_data_processed/pop.dta", keep(3) nogen	
rename country_code code
merge m:1 code using "$overview_data_processed/countrycodes_current.dta", nogen
gen pop_nomiss = pop if vuln_any != .
bys ida_group_current: egen total_pop = sum(pop)								// Representative for IDA and non-IDA
bys ida_group_current: egen total_pop_nomiss = sum(pop_nomiss)
gen share_pop_nomiss = total_pop_nomiss / total_pop
drop total_pop  total_pop_nomiss share_pop_nomiss
drop if year == .
* Aggregate to ida level
collapse (sum) pop_exp_any vuln_any pop_ev, by(ida_group_current year)

save "$overview_data_processed/exposure_vulnerability_incg_panel.dta", replace