********************************************************************************
/*
Created by: Nishant Yonzan
Date: 		05/09/2024
Purpose: 	Growth rates for forecasts using GEP June 2024.
*/
********************************************************************************

********************
*** PIP GDP DATA ***
********************
use "${rawpath}piptables_gdp.dta", clear
destring value, ignore("NA") replace 
keep if data_level=="national"
drop data_level
ren country_code code
ren value gdppc_pip
bys code (year): gen gdppc_pip_growth = (gdppc_pip/gdppc_pip[_n-1] - 1) * 100


// Historical (2010-2019) growth rates for projection
gen lngdp = ln(gdppc_pip)
gen gdppc_pip_hist = .
qui levelsof code
foreach cd in `r(levels)' {
	clear results
	capture reg lngdp year if code=="`cd'" & inrange(year,2010,2019) 
	capture replace gdppc_pip_hist = exp(_b[year])-1 if code=="`cd'"
}
replace gdppc_pip_hist = gdppc_pip_hist * 100


keep code year gdppc_pip_growth gdppc_pip_hist

label var gdppc_pip_hist   "PIP GDP/capita country historical growth 2010-2019"
label var gdppc_pip_growth "PIP GDP/capita growth"
isid code year

tempfile gdp_pip
save    `gdp_pip'

********************
*** WDI GDP DATA ***
********************
wbopendata, indicator(NY.GDP.PCAP.KN) long clear
keep countrycode year ny*
rename countrycode code
rename ny_gdp_pcap_kn gdppc_wdi_lcu
// Only retaining countries in the WB universe
merge m:1 code using "${intermediatepath}PIP_countries_202409.dta", keepusing(code) keep(3) nogen
bys code (year): gen  gdppc_wdi_growth = (gdppc_wdi_lcu/gdppc_wdi_lcu[_n-1] - 1) * 100
keep code year 		  gdppc_wdi_growth
label var 			  gdppc_wdi_growth "WDI GDP/capita growth"
isid code year

tempfile gdp_wdi
save    `gdp_wdi', replace

*********************************
*** OCTOBER 2019 WEO GDP DATA ***
*********************************
import excel "${rawpath}GDP_WEO_2019_10.xlsx", sheet("WEO_Data") firstrow clear
drop if missing(Country)
keep if Units=="National currency"
drop Country Subject Scale CountrySeries Estimates Units
foreach var of varlist G-AY {
	local yr: variable label `var'
	ren `var' gdppc_weo_oct19`yr' 
	replace   gdppc_weo_oct19`yr' = "" if gdppc_weo_oct19`yr'=="n/a"
	destring  gdppc_weo_oct19`yr', replace
}
rename ISO code
replace code = "XKX" if code=="KOS"
// Only retaining countries in the WB universe
merge m:1 code using "${intermediatepath}PIP_countries_202409.dta", keepusing(code) keep(3) nogen
reshape long gdppc_weo_oct19, i(code) j(year)
bys code (year): gen gdppc_weo_oct19_growth = (gdppc_weo_oct19/gdppc_weo_oct19[_n-1] - 1) * 100
keep code year 		 gdppc_weo_oct19_growth
label var 			 gdppc_weo_oct19_growth "WEO GDP/capita 2019m10 growth"
isid code year

tempfile gdp_weo_2019_10
save    `gdp_weo_2019_10'

*********************************
*** April 2024 WEO GDP DATA ***
*********************************
/*
https://www.imf.org/en/Publications/WEO/weo-database/2024/April/download-entire-database
Turkministan/Taiwan do not have growth data in WDI post 2020 and no growth projections in GEP/MPO
So using the April 2024 WEO for just that period mostly for these countries and other with missing growth in a year or two, see below.
*/
import delimited "${rawpath}GDP_WEO_2024_04.xls",  clear
drop if missing(country)
keep if units=="National currency" & subjectdescriptor=="Gross domestic product per capita, constant prices"
drop country subject* scale countryseries* estimates units weo*
foreach var of varlist v* {
	local yr: variable label `var'
	if "`yr'" == "" {
		drop `var'
		continue
	}
	ren `var' gdppc_weo_apr24`yr' 
	replace   gdppc_weo_apr24`yr' = "" if inlist(gdppc_weo_apr24`yr',"n/a","--")
	destring  gdppc_weo_apr24`yr', replace ignore(",")
}
rename iso code
replace code = "XKX" if code=="UVK"
replace code = "PSE" if code=="WBG"
// Only retaining countries in the WB universe
merge m:1 code using "${intermediatepath}PIP_countries_202409.dta", keepusing(code) keep(3) nogen
reshape long gdppc_weo_apr24, i(code) j(year)
bys code (year): gen gdppc_weo_apr24_growth = (gdppc_weo_apr24/gdppc_weo_apr24[_n-1] - 1) * 100
keep code year 		 gdppc_weo_apr24_growth
label var 			 gdppc_weo_apr24_growth "WEO GDP/capita 2024m04 growth"
isid code year

tempfile gdp_weo_2024_04
save    `gdp_weo_2024_04'


*********************************
*** JANUARY 2020 GEP GDP DATA ***
*********************************
// using the data provided by Cedric on June 1st 2021 with CY growth forecasts
import excel "${rawpath}GDP_GEP_2021_06.xlsx", sheet("January 2020 GEP") cellrange(A1) firstrow clear
drop if missing(B)
gen code = substr(A,1,3)
drop A
tostring B-F, replace
foreach var of varlist B-F {
	local yr: variable label `var'
	rename `var' gdp_gep_jan20_growth`yr'
	replace 	 gdp_gep_jan20_growth`yr' = "" if gdp_gep_jan20_growth`yr'==".."
}
destring, replace

// Only keep 218 WB economies
merge m:1 code using "${intermediatepath}PIP_countries_202409.dta", keepusing(code) keep(3) nogen
reshape long gdp_gep_jan20_growth, i(code) j(year)
// Right now the data is not per capita. They use UN median variant forecasts. Merge with this data to turn it into per capita. 
merge 1:1 code year 		 using "${intermediatepath}UNpop1950-2100_MediumJuly2022.dta", nogen keep(1 3)
merge 1:1 code year coverage using "${intermediatepath}Population_202409", nogen keep(1 3) 
keep if coverage == "national" 
replace unpop = pop if missing(unpop)
bysort code (year): gen gdppc_gep_jan20_growth = ((1+gdp_gep_jan20_growth/100)*unpop[_n-1]/unpop-1)*100
keep code year  		gdppc_gep_jan20_growth
label var 				gdppc_gep_jan20_growth "GEP GDP/capita 2020m01 growth"
isid code year

tempfile gdp_gep_2020_01
save    `gdp_gep_2020_01'


******************************
*** June 2024 GEP GDP DATA ***
******************************

import excel using ///
	"${rawpath}GDP_GEP_2024_06_internal.xlsx", cellrange(A2) firstrow clear
ren Countrycode code
drop Countryname
drop if missing(code) | strlen(code)>50	
tostring C-H, replace
foreach var of varlist C-H {
	local yr: variable label `var'
	local yr = substr("`yr'",1,4)
	ren `var' gdp_gep_june24_growth`yr'
	replace   gdp_gep_june24_growth`yr' = "" if gdp_gep_june24_growth`yr'==".."
}

// Only keep 218 WB economies
merge m:1 code using "${intermediatepath}PIP_countries_202409.dta", keepusing(code) nogen keep(3)
destring 	 gdp_gep_june24_growth*, replace	
reshape long gdp_gep_june24_growth, i(code) j(year)

// Right now the data is not per capita. They use UN median variant forecasts. Merge with this data to turn it into per capita. 
merge 1:1 code year 		 using "${intermediatepath}UNpop1950-2100_MediumJuly2022.dta", nogen keep(1 3)
merge 1:1 code year coverage using "${intermediatepath}Population_202409", nogen keep(1 3)
keep if coverage == "national"
replace unpop = pop if missing(unpop)
bysort code (year): gen gdppc_gep_june24_growth = ((1+gdp_gep_june24_growth/100)*unpop[_n-1]/unpop-1)*100
keep code year  		gdppc_gep_june24_growth
label var 				gdppc_gep_june24_growth "GEP GDP/capita 2024m06 growth"
isid code year
compress

tempfile gdp_gep_2024_06
save 	`gdp_gep_2024_06'


*******************************
*** March 2024 MPO GDP DATA ***
*******************************
// use data from link downloaded 04/04/2024
*use "\\wurepliprdfs01\gpvfile\gpv\Knowledge_Learning\Pov Projection\Central Team\MFM-allvintages.dta" , clear	
*save "${mar24nowcast}02-rawdata/POV_MFM_gdp_elas_Mar24.dta", replace
use "${rawpath}POV_MFM_gdp_elas_Mar24.dta", clear				
keep if date=="Mar-29-2024"
bysort countryname (year): gen  gdppc_mpo_mar24_growth = ((gdpconstantppp/gdpconstantppp[_n-1])*pop[_n-1]/pop-1)*100
replace 						gdppc_mpo_mar24_growth = growth_gdppc*100 if missing(gdppc_mpo_mar24_growth)
keep countrycode year 			gdppc_mpo_mar24_growth
ren countrycode code
label var 						gdppc_mpo_mar24_growth "MPO GDP/capita 2024m03 growth"
isid code year

tempfile gdp_mpo_2024_03
save    `gdp_mpo_2024_03'


***************
*** MERGING ***
***************
use `gdp_pip', clear
merge 1:1 code year using `gdp_wdi', nogen
merge 1:1 code year using `gdp_weo_2019_10', nogen
merge 1:1 code year using `gdp_gep_2020_01', nogen
merge 1:1 code year using `gdp_mpo_2024_03', nogen
merge 1:1 code year using `gdp_weo_2024_04', nogen
merge 1:1 code year using `gdp_gep_2024_06', nogen


// Tag for country has data in PIP
merge m:1 code using "${intermediatepath}PIP_countries_202409.dta", keepusing(region_pip) nogen 
keep if inrange(year,2010,2029)													// growth rates and forecasts available upto 2028, then use historical

*************************************************
*** CREATING PREFERED GDP GROWTH RATES SERIES ***
*************************************************
***********************************
/* 
Use PIP growth rates upto 2024 appended with WDI growth rates
Replace 2025-2029 with forecasted growth rates - June 2024 GEP + March 2024 MPO  + April 2024 WEO
*/
***********************************

*** PIP HISTORICAL GROWTH DATA UP TO 2024 ***
/* 
use the most recent gdp numbers from PIP (for historical data upto 2024)
replace missing with WDI or GEP June 2024, or MPO March 2024, or WEO April 2024  historical growth if avaible
WDI has GDP data upto 2022 + nowcasted GDP upto 2024 in PIP 
*/
gen 	historical = .
replace historical = gdppc_pip_growth/100 		if year<=2024
replace historical = gdppc_wdi_growth/100 		if missing(historical) & year<=2024
replace historical = gdppc_gep_june24_growth/100 if missing(historical) & year<=2024 // estimated values if missing actual values
replace historical = gdppc_mpo_mar24_growth/100 if missing(historical) & year<=2024 // estimated values if missing actual values
replace historical = gdppc_weo_apr24_growth/100 if missing(historical) & year<=2024 // estimated values if missing actual values


*** WELFARE PROJECTIONS FOR 2025-2028 ***
/* 
Use June 2024 GEP; next use Mar 2024 MPO; next use Apr 2024 WEO
*/
gen 	gbaseline = historical 					if year<=2024
replace gbaseline = gdppc_gep_june24_growth/100 if missing(gbaseline) & inrange(year,2025,2026) // June 2024 GEP has growth forecasts upto 2026
replace gbaseline = gdppc_mpo_mar24_growth/100  if missing(gbaseline) & inrange(year,2025,2026) // Mar 2024 MPO has growth forecasts upto 2026
replace gbaseline = gdppc_weo_apr24_growth/100  if missing(gbaseline) & inrange(year,2025,2029) // April 2024 WEO has growth forecasts upto 2029



*** PRECOVID FORECAST FOR 2021-2022 *** 
/* 
use GEP jan 2020; append this oct 2019 WEO; upto 2019 growth is forecasted not realized
Syria is missing precovid growth rates for 2020-2022
*/
gen 	gprecovid = historical 					if year<=2019
replace gprecovid = gdppc_gep_jan20_growth/100  if inrange(year,2020,2022)
replace gprecovid = gdppc_weo_oct19_growth/100  if missing(gprecovid) & inrange(year,2020,2024)


***********************************

*** Extend series to 2050 ***
levelsof code 
foreach ccc in `r(levels)' {										// take the last year of each country and add rows to fillin to 2050
	qui sum year if code=="`ccc'"	
	local   expandyr = (2050 + 1) - `r(max)' 		
	expand `expandyr' if year==`r(max)' & code=="`ccc'", gen(tag)
	
	replace year = `r(max)'+1 if tag==1 & code=="`ccc'"				// renumber the years for the added rows				
	bys tag (year): replace year = year[_n-1]+1 if _n!=1 & tag==1 & code=="`ccc'"
	
	foreach var of varlist gdppc* historical gbaseline gprecovid {	// set the values of the added rows to missing if they are repeated from last row
		replace `var' = . if tag==1	
	}
	
	drop tag
}

bys code (year): replace gdppc_pip_hist = gdppc_pip_hist[_n-1] if missing(gdppc_pip_hist)


*** Replace precovid 2023 onwards, and covid 2030 onwards with average annual growth values for all countries, but also if missing before 2029 ***
*** fillin growth rates that are not in PIP and missing growth rates ***
*count if missing(gbaseline) & inPIP=="Yes"

foreach var of varlist gprecovid gbaseline {
	replace `var' = gdppc_pip_hist/100 if inrange(year,2020,2050) & missing(`var') 
}

*** ID for the source of the growth data for baseline series ***
gen 	source = "PIP" 		  if round(gbaseline,0.00001)==round(gdppc_pip_growth/100,0.00001) & !missing(gdppc_pip_growth)
replace source = "WDI" 		  if round(gbaseline,0.00001)==round(gdppc_wdi_growth/100,0.00001) & missing(source) & !missing(gdppc_wdi_growth)
replace source = "GEP2024m06" if round(gbaseline,0.00001)==round(gdppc_gep_june24/100,0.00001) & missing(source) & !missing(gdppc_gep_june24_growth)
replace source = "MPO2024m03" if round(gbaseline,0.00001)==round(gdppc_mpo_mar24/100,0.00001)  & missing(source) & !missing(gdppc_mpo_mar24_growth)
replace source = "WEO2024m04" if round(gbaseline,0.00001)==round(gdppc_weo_apr24/100,0.00001)  & missing(source) & !missing(gdppc_weo_apr24_growth)
replace source = "HistAvg" 	  if missing(source) & !missing(gdppc_pip_hist)

keep code region_pip year gbaseline* gprecovid* source
keep if inrange(year,2020,2050)

***********************************

label var gbaseline "GDP/capita growth, PIP + GEP2024m06 appended with WEO2024m04 and MPO2024m03; historical 2010-19 & 2030+"
label var gprecovid "GDP/capita growth, PIP + GEP2020m01 appended with WEO2019m10; historical 2010-19 & 2025+"
label var source 	"Source of gbaseline growth rates"

compress
format g* %9.4f
sort code year
order code region_pip year source gprecovid gbaseline*
save "${intermediatepath}GDP_GEP202406.dta", replace

********************************************************************************
exit
