********************************************************************************
*** take the welfare vector in 2024 and project to 2030 using 1% or 2% annual reduction inequality
*** growth current set at 2% annual per country
********************************************************************************

global stryear = 2024
global endyear = 2050

global alpha "-0.01 -0.02" // inequality change scenario: -1%, -2%


cap erase "GlobalProjectedDist_${stryear}_${endyear}.dta"
cap erase "povout.dta"

********************************************************************************

// Most recent data to project
use "${rawpath}GlobalDist1000bins_1990_2024_3pr24.dta", clear				
keep if year==$stryear
ren welf welf$stryear
keep code quantile welf$stryear


// add growth rates up to 2050
// pass-through
preserve
use "${intermediatepath}GDP_GEP202406.dta", clear						
gen gpct2 = 0.02									// 2% growth all countries
gen gpct4 = 0.04									// 4% growth all countries
merge m:1 code using "${intermediatepath}PIP_countries_202409.dta", keep(1 3) keepusing(country welfare_type inPIP) nogen
foreach v of varlist gpct* {
	replace `v' = `v' * 0.7 if inlist(welfare_type,1) // consumption gets 0.7; welfare_type==2 gets pass-through of 1
}
keep if inrange(year,2025,2050)
keep country code year inPIP gpct2					// use 2% growth rate
ren g* growth_
reshape wide growth_, i(code) j(year)
tempfile grrates
save `grrates', replace
restore


merge m:1 code using `grrates', nogen 
keep if inPIP=="Yes"

tempfile datasofar
save 	`datasofar'

local y1 = $stryear + 1

levelsof code, local(levels) 
foreach ccc of local levels {

	foreach a in $alpha { 
						
		local b = -100 * `a'												// to label inequality scenario
						
		forval yr = `y1'/$endyear {
			
			qui local yr0 = `yr' - 1
			
			qui use "`datasofar'", clear
			
			qui keep if code=="`ccc'" 
			
			di "******* code: `ccc'; alpha: `a'; year: `yr' *******" 		// to keep track of the loop
			
			qui cap gen welf`yr0' = welf`yr0'_`b'
			
			qui {
				
				keep code quantile welf`yr0' growth_`yr' 
				
				sum growth_`yr'
				local grw = r(mean)
				
				sum welf`yr0' if quantile<=400, d									// need bottom 40 share for spp; spp goes into povsim not gini
				local b40sum = r(sum)
				sum welf`yr0', d
				local totalsum = r(sum)
				local b40share = `b40sum'/`totalsum'

				local alpha = `a'
				
				local spp = -`alpha'*(1+`grw')*(0.4/`b40share'-1)					// shared prosperity premium used for povsim instead of gini

				povsim welf`yr0', type(welfare) gic(l) growth(`grw') premium(`spp') name(povout) repetitions(1) replace
				
				use povout, clear												// cleanup out and save the latest year welfare vector
				keep welfare1 mtile1
				ren mtile1 		quantile
				ren welfare1 	welf`yr'_`b'
				gen code	 = "`ccc'"
				
				merge 1:1 code quantile using "`datasofar'", nogen update	// add the latest year welfare vector to use a baseline for next year
				save "`datasofar'", replace
				save "GlobalProjectedDist_${stryear}_${endyear}.dta", replace
			}
		}

	}
}

******************
*** Clean data ***
******************

use "GlobalProjectedDist_${stryear}_${endyear}.dta", clear
keep country code inPIP quantile welf* 
drop *2024
order country code inPIP quantile welf* 
reshape long welf, i(country code inPIP quantile) j(y) string 
gen year 	 = substr(y, 1, 4)
gen scenario = substr(y,-1,.)
destring year, replace
drop y



// replace welfare with regional average for countries not in pip (and missing precovid and welfare values)
// replace missing values with regional average
preserve
use "${intermediatepath}PIP_countries_202409.dta", clear					// need 1000 bins, 26 years (2025-50), 2 scenario
keep if inPIP =="No" & region=="SSA"
keep country code inPIP
expand 1000
bys code: gen quantile = _n
expand 26
gen year = 2025
bys code quantile: replace year = year + _n - 1
expand 2
bys code quantile year: gen 	scenario="1" if _n==1
bys code quantile year: replace scenario="2" if _n==2
tempfile nonpip
save 	`nonpip'
restore 

preserve
use "${intermediatepath}Population_202409", clear
keep if coverage=="national"
keep code year pop
replace pop = pop/1000															// population per 1000 bin
keep if inrange(year,2025,2050)
tempfile popu
save 	`popu', replace
restore

append using `nonpip'
merge m:1 code year using `popu', nogen keep(3)
merge m:1 code 		using "${intermediatepath}PIP_countries_202409.dta", nogen keepusing(region_pip) keep(3)

reshape wide welf, i(country code inPIP quantile region_pip year pop) j(scenario) string
reshape wide welf1 welf2 pop, i( country code inPIP quantile region_pip) j(year)


foreach v of varlist welf* {
	local yr = substr("`v'",-4,.)
	local sc = substr("`v'",-5,1)
	replace `v' = . if inPIP=="No"
	
	preserve																	// 1000 regional bins to calculate average per regional bin
	drop quantile
	egen id = group(region_pip)												
	egen quantile = xtile(`v') if !missing(`v'), nq(1000) by(id) weight(pop`yr')
	collapse `v' [aw=pop`yr'], by(region_pip quantile)
	drop if missing(`v')
	ren `v' `v'_m
	tempfile regional_average
	save 	`regional_average'
	restore

	merge m:1 region_pip quantile using `regional_average', nogen

	bys region_pip (quantile): replace `v'_m = `v'_m[_n-1] if missing(`v'_m)	// same average so missing quantile value using xtile
	replace `v' = `v'_m if missing(`v')											// replace missing values by regional average for each bin
	drop `v'_m
}

reshape long welf1 welf2 pop, i(country code inPIP quantile region_pip) j(year) 
reshape long welf, i(country code inPIP quantile region_pip year pop) j(scenario) 

tostring scenario, replace
replace scenario = "2pct_gini1" if scenario=="1"
replace scenario = "2pct_gini2" if scenario=="2"

order country code region_pip inPIP quantile year scenario pop welf

label var year 		"Year"
label var country	"Country names"
label var code 		"Country ISO3c code"
label var region_pip "PIP regions"
label var inPIP 	"Microdata support available in PIP in 2022?"
label var quantile 	"Bin number"
label var pop 		"Population of bin in millions"
label var scenario  "Decline in inequality, %"
label var welf 		"Daily per capita 2017 PPP$"

compress 
save "${outputpath}GlobalDist1000bins_2025_2050_3pr24_ineqscenario_constant2pctgrowth.dta", replace				

cap erase "GlobalProjectedDist_${stryear}_${endyear}.dta"
cap erase "povout.dta"

********************************************************************************
exit