********************************************************************************
/*
Country-level and regional FGT measures for 3PR 2024 nowcasts up to 2030 and 2050.
*/
********************************************************************************

********************************
*** HISTORICAL DATA FROM PIP ***
********************************

// country data
use "${intermediatepath}lineup_allcountries_PIP_202409", clear
drop if year<1990
keep if inlist(poverty_line,2.15,3.65,6.85)
gen inPIP = (missing==0)

replace inPIP=1 if country_code=="ARG" 											// Convert ARG to national aggregate
collapse headcount  poverty_gap poverty_severity (rawsum) poorpop (rawsum) pop [w=pop], ///
	by(country_name country_code inPIP year poverty_line)

foreach v of varlist headcount poverty_gap poverty_severity {
	replace `v' = `v' * 100
}

ren country_name country
ren country_code code 
ren poverty_line povertyline
ren headcount 		 fgt0
ren poverty_gap 	 fgt1
ren poverty_severity fgt2

gen scenario="historical"

tempfile historicalcountry
save 	`historicalcountry'


// global data
use "${rawpath}pip_aggregate_v20240802.dta", clear
drop if year<1990
keep if inlist(poverty_line,2.15,3.65,6.85)
keep if region_code=="WLD"

foreach v of varlist headcount poverty_gap poverty_severity {
	replace `v' = `v' * 100
}

foreach v of varlist population pop_in_poverty {
	format %15.6f `v'
	replace `v' = `v'/1000000
}

ren poverty_line 	 povertyline
ren headcount 		 fgt0
ren poverty_gap 	 fgt1
ren poverty_severity fgt2
ren population 		 pop
ren pop_in_poverty	 poorpop

keep year povertyline fgt0 fgt1 fgt2 pop poorpop
gen scenario="historical"

tempfile historicalglobal
save 	`historicalglobal'


// regional data
use "${rawpath}pip_aggregate_v20240802.dta", clear
drop if year<1990
keep if inlist(poverty_line,2.15,3.65,6.85)
drop if inlist(region_code,"WLD","AFE","AFW")

foreach v of varlist headcount poverty_gap poverty_severity {
	replace `v' = `v' * 100
}

foreach v of varlist population pop_in_poverty {
	format %15.6f `v'
	replace `v' = `v'/1000000
}

ren region_code		 region_pip
ren poverty_line 	 povertyline
ren headcount 		 fgt0
ren poverty_gap 	 fgt1
ren poverty_severity fgt2
ren population 		 pop
ren pop_in_poverty	 poorpop

keep year povertyline region_pip fgt0 fgt1 fgt2 pop poorpop
gen scenario="historical"

tempfile historicalregion
save 	`historicalregion'


*****************************
*** PROJECTED BINNED DATA ***
*****************************


use "${outputpath}GlobalDist1000bins_2025_2050_3pr24.dta", clear
gen scenario = "current_forecast"
append using "${outputpath}GlobalDist1000bins_2025_2050_3pr24_ineqscenario_constant2pctgrowth.dta"
append using "${outputpath}GlobalDist1000bins_2025_2050_3pr24_constantgrowth.dta"

reshape wide welf, i(country code region_pip inPIP year quantile pop) j(scenario) string

foreach var of varlist welf* {

	foreach l in 2.15 3.65 6.85 {
		local ll = `l' * 100
		local vv = substr("`var'",5,.)
		gen fgt0`vv'_`ll' = `var'<`l'
		gen fgt1`vv'_`ll' = (`l' - `var')/`l'
		gen fgt2`vv'_`ll' = ((`l' - `var')/`l')^2
		
		replace fgt1`vv'_`ll' = 0 if fgt0`vv'_`ll'==0
		replace fgt2`vv'_`ll' = 0 if fgt0`vv'_`ll'==0
	}

}

collapse fgt* (rawsum) pop [aw=pop], by(country code year ) 					// country estimates

reshape long fgt0 fgt1 fgt2, i(country code pop year) j(scenario) string
gen povertyline = substr(scenario,-3,.)
replace scenario = substr(scenario,1,strlen(scenario) - 4)

destring povertyline, replace
recast double povertyline 
replace povertyline = povertyline/100

gen inPIP = 0
gen poorpop = fgt0 * pop
foreach v of varlist fgt* {
	replace `v' = `v' * 100
}

append using `historicalcountry'												// add historical data

*******************************
*** APPEND AND DATA CLEANUP ***
*******************************

// add country names and WB regions
merge m:1 code using "${intermediatepath}IncGroupFY24.dta", nogen

keep  country code region_wb region_pip inPIP incgroup ida fcv year povertyline scenario pop fgt* poorpop
order country code region_wb region_pip inPIP incgroup ida fcv year povertyline scenario pop fgt* poorpop
sort  code povertyline year scenario
format fgt* poor* %9.2f

label var country 	  "World Bank economies"
label var inPIP 	  "Microdata available in PIP in 2019?"
label var incgroupFY24 "WB FY2024 income classification, "
label var idaFY24	  "WB FY2024 IDA classification"
label var fcvFY24	  "WB FY2024 FCV classification"
label var year		  "Year"
label var povertyline "Poverty line, 2017 PPP USD"
label var scenario 	  "Growth scenario"
label var pop 		  "Population in millions"
label var fgt0   	  "Poverty headcount rate, %"
label var fgt1   	  "Poverty gap ratio, %"
label var fgt2   	  "Poverty severity x100"
label var poorpop 	  "Number of poor, millions"


**************************
*** SAVE VARIOUS FILES ***
**************************

*** 1990-2050 historical, nowcast and forecast ***
compress
save "${outputpath}FGTcountry_1990_2050_3pr24.dta", replace

// global
preserve 
collapse fgt* (rawsum) poorpop (rawsum) pop [aw=pop], by(year povertyline scenario)
drop if year<2025
append using `historicalglobal'
label var year		  "Year"
label var povertyline "Poverty line, 2017 PPP USD"
label var scenario 	  "Growth scenario"
label var fgt0   	  "Poverty headcount rate, %"
label var fgt1   	  "Poverty gap ratio, %"
label var fgt2   	  "Poverty severity x100"
label var pop 	  	  "Population, millions"
label var poorpop 	  "Number of poor, millions"
sort year povertyline
save "${outputpath}FGTglobal_1990_2050_3pr24.dta", replace
restore

// regional 
preserve 
collapse fgt* (rawsum) poorpop (rawsum) pop [aw=pop], by(year povertyline scenario region_pip)
merge m:1 year region_pip povertyline using `historicalregion', nogen update replace
append using `historicalregion'
label var year		  "Year"
label var region_pip  "Region, PIP"
label var povertyline "Poverty line, 2017 PPP USD"
label var scenario 	  "Growth scenario"
label var fgt0   	  "Poverty headcount rate, %"
label var fgt1   	  "Poverty gap ratio, %"
label var fgt2   	  "Poverty severity x100"
label var pop 	  	  "Population, millions"
label var poorpop 	  "Number of poor, millions"
sort year povertyline
save "${outputpath}FGTregion_1990_2050_3pr24.dta", replace
restore


********************************************************************************

exit
